# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.block_storage.v3 import backup as _backup
from openstack.block_storage.v3 import volume as _volume
from openstack.tests.functional.block_storage.v3 import base


class TestBackup(base.BaseBlockStorageTest):
    def setUp(self):
        super().setUp()

        if not self.user_cloud.has_service('object-store'):
            self.skipTest('Object service is requred, but not available')

        self.VOLUME_NAME = self.getUniqueString()
        self.VOLUME_ID = None
        self.BACKUP_NAME = self.getUniqueString()
        self.BACKUP_ID = None

        volume = self.user_cloud.block_storage.create_volume(
            name=self.VOLUME_NAME, size=1
        )
        self.user_cloud.block_storage.wait_for_status(
            volume,
            status='available',
            failures=['error'],
            interval=5,
            wait=self._wait_for_timeout,
        )
        assert isinstance(volume, _volume.Volume)
        self.VOLUME_ID = volume.id

        backup = self.user_cloud.block_storage.create_backup(
            name=self.BACKUP_NAME, volume_id=volume.id, is_incremental=False
        )
        self.user_cloud.block_storage.wait_for_status(
            backup,
            status='available',
            failures=['error'],
            interval=5,
            wait=self._wait_for_timeout,
        )
        assert isinstance(backup, _backup.Backup)
        self.assertEqual(self.BACKUP_NAME, backup.name)
        self.BACKUP_ID = backup.id

    def tearDown(self):
        sot = self.user_cloud.block_storage.delete_backup(
            self.BACKUP_ID, ignore_missing=False
        )
        sot = self.user_cloud.block_storage.delete_volume(
            self.VOLUME_ID, ignore_missing=False
        )
        self.assertIsNone(sot)
        super().tearDown()

    def test_get(self):
        sot = self.user_cloud.block_storage.get_backup(self.BACKUP_ID)
        self.assertEqual(self.BACKUP_NAME, sot.name)
        self.assertEqual(False, sot.is_incremental)

    def test_create_metadata(self):
        metadata_backup = self.user_cloud.block_storage.create_backup(
            name=self.getUniqueString(),
            volume_id=self.VOLUME_ID,
            metadata=dict(foo="bar"),
        )
        self.user_cloud.block_storage.wait_for_status(
            metadata_backup,
            status='available',
            failures=['error'],
            interval=5,
            wait=self._wait_for_timeout,
        )
        self.user_cloud.block_storage.delete_backup(
            metadata_backup.id, ignore_missing=False
        )

    def test_create_incremental(self):
        incremental_backup = self.user_cloud.block_storage.create_backup(
            name=self.getUniqueString(),
            volume_id=self.VOLUME_ID,
            is_incremental=True,
        )
        self.user_cloud.block_storage.wait_for_status(
            incremental_backup,
            status='available',
            failures=['error'],
            interval=5,
            wait=self._wait_for_timeout,
        )
        self.assertEqual(True, incremental_backup.is_incremental)
        self.user_cloud.block_storage.delete_backup(
            incremental_backup.id, ignore_missing=False
        )
        self.user_cloud.block_storage.wait_for_delete(incremental_backup)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from openstack.network.v2 import qos_policy
from openstack.tests.unit import base


EXAMPLE = {
    'id': 'IDENTIFIER',
    'description': 'QoS policy description',
    'name': 'qos-policy-name',
    'shared': True,
    'project_id': '2',
    'rules': [uuid.uuid4().hex],
    'is_default': False,
    'tags': ['3'],
}


class TestQoSPolicy(base.TestCase):
    def test_basic(self):
        sot = qos_policy.QoSPolicy()
        self.assertEqual('policy', sot.resource_key)
        self.assertEqual('policies', sot.resources_key)
        self.assertEqual('/qos/policies', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = qos_policy.QoSPolicy(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['rules'], sot.rules)
        self.assertEqual(EXAMPLE['is_default'], sot.is_default)
        self.assertEqual(EXAMPLE['tags'], sot.tags)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import quota as _quota
from openstack.tests.functional import base


class TestQuota(base.BaseFunctionalTest):
    def setUp(self):
        super().setUp()

        if not self.operator_cloud:
            self.skipTest("Operator cloud required for this test")

        self.project = self.create_temporary_project()

    def test_quota(self):
        # update quota

        quota = self.operator_cloud.network.update_quota(
            self.project.id, networks=123456789
        )
        self.assertIsInstance(quota, _quota.Quota)
        self.assertEqual(quota.networks, 123456789)

        # retrieve details of the (updated) quota

        quota = self.operator_cloud.network.get_quota(self.project.id)
        self.assertIsInstance(quota, _quota.Quota)
        self.assertEqual(quota.networks, 123456789)

        # retrieve quota defaults

        defaults = self.operator_cloud.network.get_quota_default(
            self.project.id
        )
        self.assertIsInstance(defaults, _quota.QuotaDefault)
        self.assertNotEqual(defaults.networks, 123456789)

        # list quotas

        quotas = list(self.operator_cloud.network.quotas())
        self.assertIn(self.project.id, [x.project_id for x in quotas])

        # revert quota

        ret = self.operator_cloud.network.delete_quota(self.project.id)
        self.assertIsNone(ret)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from keystoneauth1 import adapter

from openstack.baremetal.v1 import port
from openstack.tests.unit import base


FAKE = {
    "address": "11:11:11:11:11:11",
    "created_at": "2016-08-18T22:28:49.946416+00:00",
    "description": "Physical network",
    "extra": {},
    "internal_info": {},
    "is_smartnic": True,
    "links": [
        {"href": "http://127.0.0.1:6385/v1/ports/<PORT_ID>", "rel": "self"},
        {"href": "http://127.0.0.1:6385/ports/<PORT_ID>", "rel": "bookmark"},
    ],
    "local_link_connection": {
        "port_id": "Ethernet3/1",
        "switch_id": "0a:1b:2c:3d:4e:5f",
        "switch_info": "switch1",
    },
    "name": "port_name",
    "node_uuid": "6d85703a-565d-469a-96ce-30b6de53079d",
    "portgroup_uuid": "e43c722c-248e-4c6e-8ce8-0d8ff129387a",
    "pxe_enabled": True,
    "updated_at": None,
    "uuid": "d2b30520-907d-46c8-bfee-c5586e6fb3a1",
}


class TestPort(base.TestCase):
    def test_basic(self):
        sot = port.Port()
        self.assertIsNone(sot.resource_key)
        self.assertEqual('ports', sot.resources_key)
        self.assertEqual('/ports', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertEqual('PATCH', sot.commit_method)

    def test_instantiate(self):
        sot = port.PortDetail(**FAKE)
        self.assertEqual(FAKE['uuid'], sot.id)
        self.assertEqual(FAKE['address'], sot.address)
        self.assertEqual(FAKE['created_at'], sot.created_at)
        self.assertEqual(FAKE['extra'], sot.extra)
        self.assertEqual(FAKE['internal_info'], sot.internal_info)
        self.assertEqual(FAKE['is_smartnic'], sot.is_smartnic)
        self.assertEqual(FAKE['links'], sot.links)
        self.assertEqual(
            FAKE['local_link_connection'], sot.local_link_connection
        )
        self.assertEqual(FAKE['name'], sot.name)
        self.assertEqual(FAKE['node_uuid'], sot.node_id)
        self.assertEqual(FAKE['portgroup_uuid'], sot.port_group_id)
        self.assertEqual(FAKE['pxe_enabled'], sot.is_pxe_enabled)
        self.assertEqual(FAKE['updated_at'], sot.updated_at)

    def test_list_conductor_groups(self):
        self.port = port.Port()
        self.session = mock.Mock(
            spec=adapter.Adapter, default_microversion=None
        )

        self.session.default_microversion = float(self.port._max_microversion)
        self.session.get.return_value.status_code = 200
        self.session.get.return_value.json.return_value = {'ports': []}

        result = list(
            self.port.list(
                self.session,
                details=False,
                conductor_groups=['group1', 'group2'],
                allow_unknown_params=True,
            )
        )
        self.assertEqual(0, len(result))
        self.session.get.assert_called_once_with(
            '/ports',
            headers={'Accept': 'application/json'},
            params={'conductor_groups': ['group1', 'group2']},
            microversion=float(self.port._max_microversion),
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class RBACPolicy(resource.Resource):
    resource_key = 'rbac_policy'
    resources_key = 'rbac_policies'
    base_path = '/rbac-policies'

    _allow_unknown_attrs_in_body = True

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'action',
        'object_id',
        'object_type',
        'project_id',
        'target_project_id',
        target_project_id='target_tenant',
    )

    # Properties
    #: ID of the object that this RBAC policy affects.
    object_id = resource.Body('object_id')
    #: The ID of the project this RBAC will be enforced.
    target_project_id = resource.Body('target_tenant')
    #: The owner project ID.
    project_id = resource.Body('project_id', alias='tenant_id')
    #: Tenant_id (deprecated attribute).
    tenant_id = resource.Body('tenant_id', deprecated=True)
    #: Type of the object that this RBAC policy affects.
    object_type = resource.Body('object_type')
    #: Action for the RBAC policy.
    action = resource.Body('action')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_compute
----------------------------------

Functional tests for image methods.
"""

import filecmp
import os
import tempfile

from openstack.tests.functional import base


class TestImage(base.BaseFunctionalTest):
    def test_create_image(self):
        test_image = tempfile.NamedTemporaryFile(delete=False)
        test_image.write(b'\0' * 1024 * 1024)
        test_image.close()
        image_name = self.getUniqueString('image')
        try:
            self.user_cloud.create_image(
                name=image_name,
                filename=test_image.name,
                disk_format='raw',
                container_format='bare',
                min_disk=10,
                min_ram=1024,
                tags=['custom'],
                wait=True,
            )
        finally:
            self.user_cloud.delete_image(image_name, wait=True)

    def test_download_image(self):
        test_image = tempfile.NamedTemporaryFile(delete=False)
        self.addCleanup(os.remove, test_image.name)
        test_image.write(b'\0' * 1024 * 1024)
        test_image.close()
        image_name = self.getUniqueString('image')
        self.user_cloud.create_image(
            name=image_name,
            filename=test_image.name,
            disk_format='raw',
            container_format='bare',
            min_disk=10,
            min_ram=1024,
            wait=True,
        )
        self.addCleanup(self.user_cloud.delete_image, image_name, wait=True)
        output = os.path.join(tempfile.gettempdir(), self.getUniqueString())
        self.user_cloud.download_image(image_name, output)
        self.addCleanup(os.remove, output)
        self.assertTrue(
            filecmp.cmp(test_image.name, output),
            "Downloaded contents don't match created image",
        )

    def test_create_image_skip_duplicate(self):
        test_image = tempfile.NamedTemporaryFile(delete=False)
        test_image.write(b'\0' * 1024 * 1024)
        test_image.close()
        image_name = self.getUniqueString('image')
        try:
            first_image = self.user_cloud.create_image(
                name=image_name,
                filename=test_image.name,
                disk_format='raw',
                container_format='bare',
                min_disk=10,
                min_ram=1024,
                validate_checksum=True,
                wait=True,
            )
            second_image = self.user_cloud.create_image(
                name=image_name,
                filename=test_image.name,
                disk_format='raw',
                container_format='bare',
                min_disk=10,
                min_ram=1024,
                validate_checksum=True,
                wait=True,
            )
            self.assertEqual(first_image.id, second_image.id)
        finally:
            self.user_cloud.delete_image(image_name, wait=True)

    def test_create_image_force_duplicate(self):
        test_image = tempfile.NamedTemporaryFile(delete=False)
        test_image.write(b'\0' * 1024 * 1024)
        test_image.close()
        image_name = self.getUniqueString('image')
        first_image = None
        second_image = None
        try:
            first_image = self.user_cloud.create_image(
                name=image_name,
                filename=test_image.name,
                disk_format='raw',
                container_format='bare',
                min_disk=10,
                min_ram=1024,
                wait=True,
            )
            second_image = self.user_cloud.create_image(
                name=image_name,
                filename=test_image.name,
                disk_format='raw',
                container_format='bare',
                min_disk=10,
                min_ram=1024,
                allow_duplicates=True,
                wait=True,
            )
            self.assertNotEqual(first_image.id, second_image.id)
        finally:
            if first_image:
                self.user_cloud.delete_image(first_image.id, wait=True)
            if second_image:
                self.user_cloud.delete_image(second_image.id, wait=True)

    def test_create_image_update_properties(self):
        test_image = tempfile.NamedTemporaryFile(delete=False)
        test_image.write(b'\0' * 1024 * 1024)
        test_image.close()
        image_name = self.getUniqueString('image')
        try:
            image = self.user_cloud.create_image(
                name=image_name,
                filename=test_image.name,
                disk_format='raw',
                container_format='bare',
                min_disk=10,
                min_ram=1024,
                wait=True,
            )
            self.user_cloud.update_image_properties(
                image=image, name=image_name, foo='bar'
            )
            image = self.user_cloud.get_image(image_name)
            self.assertIn('foo', image.properties)
            self.assertEqual(image.properties['foo'], 'bar')
        finally:
            self.user_cloud.delete_image(image_name, wait=True)

    def test_create_image_without_filename(self):
        image_name = self.getUniqueString('image')
        image = self.user_cloud.create_image(
            name=image_name,
            disk_format='raw',
            container_format='bare',
            min_disk=10,
            min_ram=1024,
            allow_duplicates=True,
            wait=False,
        )
        self.assertEqual(image_name, image.name)
        self.user_cloud.delete_image(image.id, wait=True)

    def test_get_image_by_id(self):
        test_image = tempfile.NamedTemporaryFile(delete=False)
        test_image.write(b'\0' * 1024 * 1024)
        test_image.close()
        image_name = self.getUniqueString('image')
        try:
            image = self.user_cloud.create_image(
                name=image_name,
                filename=test_image.name,
                disk_format='raw',
                container_format='bare',
                min_disk=10,
                min_ram=1024,
                wait=True,
            )
            image = self.user_cloud.get_image_by_id(image.id)
            self.assertEqual(image_name, image.name)
            self.assertEqual('raw', image.disk_format)
        finally:
            self.user_cloud.delete_image(image_name, wait=True)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
from unittest import mock
import uuid

from openstack.message.v2 import claim
from openstack.tests.unit import base

FAKE1 = {
    "age": 1632,
    "id": "576b54963990b48c644bb7e7",
    "grace": 3600,
    "limit": 10,
    "messages": [{"id": "1"}, {"id": "2"}],
    "ttl": 3600,
    "queue_name": "queue1",
}


FAKE2 = {
    "age": 1632,
    "id": "576b54963990b48c644bb7e7",
    "grace": 3600,
    "limit": 10,
    "messages": [{"id": "1"}, {"id": "2"}],
    "ttl": 3600,
    "queue_name": "queue1",
    "client_id": "OLD_CLIENT_ID",
    "project_id": "OLD_PROJECT_ID",
}


class TestClaim(base.TestCase):
    def test_basic(self):
        sot = claim.Claim()
        self.assertEqual("claims", sot.resources_key)
        self.assertEqual("/queues/%(queue_name)s/claims", sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)

    def test_make_it(self):
        sot = claim.Claim.new(**FAKE2)
        self.assertEqual(FAKE2["age"], sot.age)
        self.assertEqual(FAKE2["id"], sot.id)
        self.assertEqual(FAKE2["grace"], sot.grace)
        self.assertEqual(FAKE2["limit"], sot.limit)
        self.assertEqual(FAKE2["messages"], sot.messages)
        self.assertEqual(FAKE2["ttl"], sot.ttl)
        self.assertEqual(FAKE2["queue_name"], sot.queue_name)
        self.assertEqual(FAKE2["client_id"], sot.client_id)
        self.assertEqual(FAKE2["project_id"], sot.project_id)

    @mock.patch.object(uuid, "uuid4")
    def test_create_204_resp(self, mock_uuid):
        sess = mock.Mock()
        resp = mock.Mock()
        sess.post.return_value = resp
        resp.status_code = 204
        sess.get_project_id.return_value = "NEW_PROJECT_ID"
        mock_uuid.return_value = "NEW_CLIENT_ID"
        FAKE = copy.deepcopy(FAKE1)

        sot = claim.Claim(**FAKE1)
        res = sot.create(sess)

        url = "/queues/{queue}/claims".format(queue=FAKE.pop("queue_name"))
        headers = {
            "Client-ID": "NEW_CLIENT_ID",
            "X-PROJECT-ID": "NEW_PROJECT_ID",
        }
        sess.post.assert_called_once_with(url, headers=headers, json=FAKE)
        sess.get_project_id.assert_called_once_with()
        self.assertEqual(sot, res)

    @mock.patch.object(uuid, "uuid4")
    def test_create_non_204_resp(self, mock_uuid):
        sess = mock.Mock()
        resp = mock.Mock()
        sess.post.return_value = resp
        resp.status_code = 200
        sess.get_project_id.return_value = "NEW_PROJECT_ID"
        mock_uuid.return_value = "NEW_CLIENT_ID"
        FAKE = copy.deepcopy(FAKE1)

        sot = claim.Claim(**FAKE1)
        sot._translate_response = mock.Mock()
        res = sot.create(sess)

        url = "/queues/{queue}/claims".format(queue=FAKE.pop("queue_name"))
        headers = {
            "Client-ID": "NEW_CLIENT_ID",
            "X-PROJECT-ID": "NEW_PROJECT_ID",
        }
        sess.post.assert_called_once_with(url, headers=headers, json=FAKE)
        sess.get_project_id.assert_called_once_with()
        self.assertEqual(sot, res)
        sot._translate_response.assert_called_once_with(resp)

    def test_create_client_id_project_id_exist(self):
        sess = mock.Mock()
        resp = mock.Mock()
        sess.post.return_value = resp
        resp.status_code = 200
        FAKE = copy.deepcopy(FAKE2)

        sot = claim.Claim(**FAKE2)
        sot._translate_response = mock.Mock()
        res = sot.create(sess)

        url = "/queues/{queue}/claims".format(queue=FAKE.pop("queue_name"))
        headers = {
            "Client-ID": FAKE.pop("client_id"),
            "X-PROJECT-ID": FAKE.pop("project_id"),
        }
        sess.post.assert_called_once_with(url, headers=headers, json=FAKE)
        self.assertEqual(sot, res)

    @mock.patch.object(uuid, "uuid4")
    def test_get(self, mock_uuid):
        sess = mock.Mock()
        resp = mock.Mock()
        sess.get.return_value = resp
        sess.get_project_id.return_value = "NEW_PROJECT_ID"
        mock_uuid.return_value = "NEW_CLIENT_ID"

        sot = claim.Claim(**FAKE1)
        sot._translate_response = mock.Mock()
        res = sot.fetch(sess)

        url = "queues/{queue}/claims/{claim}".format(
            queue=FAKE1["queue_name"],
            claim=FAKE1["id"],
        )
        headers = {
            "Client-ID": "NEW_CLIENT_ID",
            "X-PROJECT-ID": "NEW_PROJECT_ID",
        }
        sess.get.assert_called_with(url, headers=headers, skip_cache=False)
        sess.get_project_id.assert_called_once_with()
        sot._translate_response.assert_called_once_with(resp)
        self.assertEqual(sot, res)

    def test_get_client_id_project_id_exist(self):
        sess = mock.Mock()
        resp = mock.Mock()
        sess.get.return_value = resp

        sot = claim.Claim(**FAKE2)
        sot._translate_response = mock.Mock()
        res = sot.fetch(sess)

        url = "queues/{queue}/claims/{claim}".format(
            queue=FAKE2["queue_name"],
            claim=FAKE2["id"],
        )
        headers = {
            "Client-ID": "OLD_CLIENT_ID",
            "X-PROJECT-ID": "OLD_PROJECT_ID",
        }
        sess.get.assert_called_with(url, headers=headers, skip_cache=False)
        sot._translate_response.assert_called_once_with(resp)
        self.assertEqual(sot, res)

    @mock.patch.object(uuid, "uuid4")
    def test_update(self, mock_uuid):
        sess = mock.Mock()
        resp = mock.Mock()
        sess.update.return_value = resp
        sess.get_project_id.return_value = "NEW_PROJECT_ID"
        mock_uuid.return_value = "NEW_CLIENT_ID"
        FAKE = copy.deepcopy(FAKE1)

        sot = claim.Claim(**FAKE1)
        res = sot.commit(sess)

        url = "queues/{queue}/claims/{claim}".format(
            queue=FAKE.pop("queue_name"),
            claim=FAKE["id"],
        )
        headers = {
            "Client-ID": "NEW_CLIENT_ID",
            "X-PROJECT-ID": "NEW_PROJECT_ID",
        }
        sess.patch.assert_called_with(url, headers=headers, json=FAKE)
        sess.get_project_id.assert_called_once_with()
        self.assertEqual(sot, res)

    def test_update_client_id_project_id_exist(self):
        sess = mock.Mock()
        resp = mock.Mock()
        sess.get.return_value = resp
        FAKE = copy.deepcopy(FAKE2)

        sot = claim.Claim(**FAKE2)
        res = sot.commit(sess)

        url = "queues/{queue}/claims/{claim}".format(
            queue=FAKE.pop("queue_name"),
            claim=FAKE["id"],
        )
        headers = {
            "Client-ID": FAKE.pop("client_id"),
            "X-PROJECT-ID": FAKE.pop("project_id"),
        }
        sess.patch.assert_called_with(url, headers=headers, json=FAKE)
        self.assertEqual(sot, res)

    @mock.patch.object(uuid, "uuid4")
    def test_delete(self, mock_uuid):
        sess = mock.Mock()
        resp = mock.Mock()
        sess.delete.return_value = resp
        sess.get_project_id.return_value = "NEW_PROJECT_ID"
        mock_uuid.return_value = "NEW_CLIENT_ID"

        sot = claim.Claim(**FAKE1)
        sot._translate_response = mock.Mock()
        sot.delete(sess)

        url = "queues/{queue}/claims/{claim}".format(
            queue=FAKE1["queue_name"],
            claim=FAKE1["id"],
        )
        headers = {
            "Client-ID": "NEW_CLIENT_ID",
            "X-PROJECT-ID": "NEW_PROJECT_ID",
        }
        sess.delete.assert_called_with(url, headers=headers)
        sess.get_project_id.assert_called_once_with()
        sot._translate_response.assert_called_once_with(resp, has_body=False)

    def test_delete_client_id_project_id_exist(self):
        sess = mock.Mock()
        resp = mock.Mock()
        sess.delete.return_value = resp

        sot = claim.Claim(**FAKE2)
        sot._translate_response = mock.Mock()
        sot.delete(sess)

        url = "queues/{queue}/claims/{claim}".format(
            queue=FAKE2["queue_name"],
            claim=FAKE2["id"],
        )
        headers = {
            "Client-ID": "OLD_CLIENT_ID",
            "X-PROJECT-ID": "OLD_PROJECT_ID",
        }
        sess.delete.assert_called_with(url, headers=headers)
        sot._translate_response.assert_called_once_with(resp, has_body=False)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import copy

import openstack.cloud
from openstack import exceptions
from openstack.tests import fakes
from openstack.tests.unit import base


# TODO(mordred): Move id and name to using a getUniqueString() value

neutron_grp_dict = fakes.make_fake_neutron_security_group(
    id='1',
    name='neutron-sec-group',
    description='Test Neutron security group',
    rules=[
        dict(
            id='1',
            port_range_min=80,
            port_range_max=81,
            protocol='tcp',
            remote_ip_prefix='0.0.0.0/0',
        )
    ],
)


nova_grp_dict = fakes.make_fake_nova_security_group(
    id='2',
    name='nova-sec-group',
    description='Test Nova security group #1',
    rules=[
        fakes.make_fake_nova_security_group_rule(
            id='2',
            from_port=8000,
            to_port=8001,
            ip_protocol='tcp',
            cidr='0.0.0.0/0',
        ),
    ],
)


class TestSecurityGroups(base.TestCase):
    def setUp(self):
        super().setUp()
        self.has_neutron = True

        def fake_has_service(*args, **kwargs):
            return self.has_neutron

        self.cloud.has_service = fake_has_service

    def test_list_security_groups_neutron(self):
        project_id = 42
        self.cloud.secgroup_source = 'neutron'
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'security-groups'],
                        qs_elements=[f"project_id={project_id}"],
                    ),
                    json={'security_groups': [neutron_grp_dict]},
                )
            ]
        )
        self.cloud.list_security_groups(filters={'project_id': project_id})
        self.assert_calls()

    def test_list_security_groups_nova(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=f'{fakes.COMPUTE_ENDPOINT}/os-security-groups?project_id=42',
                    json={'security_groups': []},
                ),
            ]
        )
        self.cloud.secgroup_source = 'nova'
        self.has_neutron = False
        self.cloud.list_security_groups(filters={'project_id': 42})

        self.assert_calls()

    def test_list_security_groups_none(self):
        self.cloud.secgroup_source = None
        self.has_neutron = False
        self.assertRaises(
            openstack.cloud.OpenStackCloudUnavailableFeature,
            self.cloud.list_security_groups,
        )

    def test_delete_security_group_neutron(self):
        sg_id = neutron_grp_dict['id']
        self.cloud.secgroup_source = 'neutron'
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'security-groups']
                    ),
                    json={'security_groups': [neutron_grp_dict]},
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'security-groups', f'{sg_id}'],
                    ),
                    status_code=200,
                    json={},
                ),
            ]
        )
        self.assertTrue(self.cloud.delete_security_group('1'))
        self.assert_calls()

    def test_delete_security_group_nova(self):
        self.cloud.secgroup_source = 'nova'
        self.has_neutron = False
        nova_return = [nova_grp_dict]
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=f'{fakes.COMPUTE_ENDPOINT}/os-security-groups',
                    json={'security_groups': nova_return},
                ),
                dict(
                    method='DELETE',
                    uri=f'{fakes.COMPUTE_ENDPOINT}/os-security-groups/2',
                ),
            ]
        )
        self.cloud.delete_security_group('2')
        self.assert_calls()

    def test_delete_security_group_neutron_not_found(self):
        self.cloud.secgroup_source = 'neutron'
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'security-groups']
                    ),
                    json={'security_groups': [neutron_grp_dict]},
                )
            ]
        )
        self.assertFalse(self.cloud.delete_security_group('10'))
        self.assert_calls()

    def test_delete_security_group_nova_not_found(self):
        self.cloud.secgroup_source = 'nova'
        self.has_neutron = False
        nova_return = [nova_grp_dict]
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=f'{fakes.COMPUTE_ENDPOINT}/os-security-groups',
                    json={'security_groups': nova_return},
                ),
            ]
        )
        self.assertFalse(self.cloud.delete_security_group('doesNotExist'))

    def test_delete_security_group_none(self):
        self.cloud.secgroup_source = None
        self.assertRaises(
            openstack.cloud.OpenStackCloudUnavailableFeature,
            self.cloud.delete_security_group,
            'doesNotExist',
        )

    def test_create_security_group_neutron(self):
        self.cloud.secgroup_source = 'neutron'
        group_name = self.getUniqueString()
        group_desc = self.getUniqueString('description')
        new_group = fakes.make_fake_neutron_security_group(
            id='2', name=group_name, description=group_desc, rules=[]
        )
        self.register_uris(
            [
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'security-groups']
                    ),
                    json={'security_group': new_group},
                    validate=dict(
                        json={
                            'security_group': {
                                'name': group_name,
                                'description': group_desc,
                            }
                        }
                    ),
                )
            ]
        )

        r = self.cloud.create_security_group(group_name, group_desc)
        self.assertEqual(group_name, r['name'])
        self.assertEqual(group_desc, r['description'])
        self.assertEqual(True, r['stateful'])

        self.assert_calls()

    def test_create_security_group_neutron_specific_tenant(self):
        self.cloud.secgroup_source = 'neutron'
        project_id = "861808a93da0484ea1767967c4df8a23"
        group_name = self.getUniqueString()
        group_desc = (
            'security group from '
            'test_create_security_group_neutron_specific_tenant'
        )
        new_group = fakes.make_fake_neutron_security_group(
            id='2',
            name=group_name,
            description=group_desc,
            project_id=project_id,
            rules=[],
        )
        self.register_uris(
            [
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'security-groups']
                    ),
                    json={'security_group': new_group},
                    validate=dict(
                        json={
                            'security_group': {
                                'name': group_name,
                                'description': group_desc,
                                'tenant_id': project_id,
                            }
                        }
                    ),
                )
            ]
        )

        r = self.cloud.create_security_group(
            group_name, group_desc, project_id
        )
        self.assertEqual(group_name, r['name'])
        self.assertEqual(group_desc, r['description'])
        self.assertEqual(project_id, r['tenant_id'])

        self.assert_calls()

    def test_create_security_group_stateless_neutron(self):
        self.cloud.secgroup_source = 'neutron'
        group_name = self.getUniqueString()
        group_desc = self.getUniqueString('description')
        new_group = fakes.make_fake_neutron_security_group(
            id='2',
            name=group_name,
            description=group_desc,
            stateful=False,
            rules=[],
        )
        self.register_uris(
            [
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'security-groups']
                    ),
                    json={'security_group': new_group},
                    validate=dict(
                        json={
                            'security_group': {
                                'name': group_name,
                                'description': group_desc,
                                'stateful': False,
                            }
                        }
                    ),
                )
            ]
        )

        r = self.cloud.create_security_group(
            group_name, group_desc, stateful=False
        )
        self.assertEqual(group_name, r['name'])
        self.assertEqual(group_desc, r['description'])
        self.assertEqual(False, r['stateful'])
        self.assert_calls()

    def test_create_security_group_nova(self):
        group_name = self.getUniqueString()
        self.has_neutron = False
        group_desc = self.getUniqueString('description')
        new_group = fakes.make_fake_nova_security_group(
            id='2', name=group_name, description=group_desc, rules=[]
        )
        self.register_uris(
            [
                dict(
                    method='POST',
                    uri=f'{fakes.COMPUTE_ENDPOINT}/os-security-groups',
                    json={'security_group': new_group},
                    validate=dict(
                        json={
                            'security_group': {
                                'name': group_name,
                                'description': group_desc,
                            }
                        }
                    ),
                ),
            ]
        )

        self.cloud.secgroup_source = 'nova'
        r = self.cloud.create_security_group(group_name, group_desc)
        self.assertEqual(group_name, r['name'])
        self.assertEqual(group_desc, r['description'])

        self.assert_calls()

    def test_create_security_group_none(self):
        self.cloud.secgroup_source = None
        self.has_neutron = False
        self.assertRaises(
            openstack.cloud.OpenStackCloudUnavailableFeature,
            self.cloud.create_security_group,
            '',
            '',
        )

    def test_update_security_group_neutron(self):
        self.cloud.secgroup_source = 'neutron'
        new_name = self.getUniqueString()
        sg_id = neutron_grp_dict['id']
        update_return = neutron_grp_dict.copy()
        update_return['name'] = new_name
        update_return['stateful'] = False
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'security-groups']
                    ),
                    json={'security_groups': [neutron_grp_dict]},
                ),
                dict(
                    method='PUT',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'security-groups', f'{sg_id}'],
                    ),
                    json={'security_group': update_return},
                    validate=dict(
                        json={
                            'security_group': {
                                'name': new_name,
                                'stateful': False,
                            }
                        }
                    ),
                ),
            ]
        )
        r = self.cloud.update_security_group(
            sg_id, name=new_name, stateful=False
        )
        self.assertEqual(r['name'], new_name)
        self.assertEqual(r['stateful'], False)
        self.assert_calls()

    def test_update_security_group_nova(self):
        self.has_neutron = False
        new_name = self.getUniqueString()
        self.cloud.secgroup_source = 'nova'
        nova_return = [nova_grp_dict]
        update_return = nova_grp_dict.copy()
        update_return['name'] = new_name

        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=f'{fakes.COMPUTE_ENDPOINT}/os-security-groups',
                    json={'security_groups': nova_return},
                ),
                dict(
                    method='PUT',
                    uri=f'{fakes.COMPUTE_ENDPOINT}/os-security-groups/2',
                    json={'security_group': update_return},
                ),
            ]
        )

        r = self.cloud.update_security_group(
            nova_grp_dict['id'], name=new_name
        )
        self.assertEqual(r['name'], new_name)
        self.assert_calls()

    def test_update_security_group_bad_kwarg(self):
        self.assertRaises(
            TypeError,
            self.cloud.update_security_group,
            'doesNotExist',
            bad_arg='',
        )

    def test_create_security_group_rule_neutron(self):
        self.cloud.secgroup_source = 'neutron'
        args = dict(
            port_range_min=-1,
            port_range_max=40000,
            protocol='tcp',
            remote_ip_prefix='0.0.0.0/0',
            remote_group_id='456',
            remote_address_group_id='1234-5678',
            direction='egress',
            ethertype='IPv6',
        )
        expected_args = copy.copy(args)
        # For neutron, -1 port should be converted to None
        expected_args['port_range_min'] = None
        expected_args['security_group_id'] = neutron_grp_dict['id']

        expected_new_rule = copy.copy(expected_args)
        expected_new_rule['id'] = '1234'
        expected_new_rule['tenant_id'] = None
        expected_new_rule['project_id'] = expected_new_rule['tenant_id']

        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'security-groups']
                    ),
                    json={'security_groups': [neutron_grp_dict]},
                ),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'security-group-rules'],
                    ),
                    json={'security_group_rule': expected_new_rule},
                    validate=dict(json={'security_group_rule': expected_args}),
                ),
            ]
        )
        new_rule = self.cloud.create_security_group_rule(
            secgroup_name_or_id=neutron_grp_dict['id'], **args
        ).to_dict(original_names=True)
        # NOTE(gtema): don't check location and not relevant properties
        # in new rule
        new_rule.pop('created_at')
        new_rule.pop('description')
        new_rule.pop('location')
        new_rule.pop('name')
        new_rule.pop('revision_number')
        new_rule.pop('tags')
        new_rule.pop('updated_at')
        new_rule.pop('if-match')
        self.assertEqual(expected_new_rule, new_rule)
        self.assert_calls()

    def test_create_security_group_rule_neutron_specific_tenant(self):
        self.cloud.secgroup_source = 'neutron'
        args = dict(
            port_range_min=-1,
            port_range_max=40000,
            protocol='tcp',
            remote_ip_prefix='0.0.0.0/0',
            remote_group_id='456',
            remote_address_group_id=None,
            direction='egress',
            ethertype='IPv6',
            project_id='861808a93da0484ea1767967c4df8a23',
        )
        expected_args = copy.copy(args)
        # For neutron, -1 port should be converted to None
        expected_args['port_range_min'] = None
        expected_args['security_group_id'] = neutron_grp_dict['id']
        expected_args['tenant_id'] = expected_args['project_id']
        expected_args.pop('project_id')

        expected_new_rule = copy.copy(expected_args)
        expected_new_rule['id'] = '1234'
        expected_new_rule['project_id'] = expected_new_rule['tenant_id']

        # This is not sent in body if == None so should not be in the
        # JSON; see SecurityGroupRule where it is removed.
        expected_args.pop('remote_address_group_id')

        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'security-groups']
                    ),
                    json={'security_groups': [neutron_grp_dict]},
                ),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'security-group-rules'],
                    ),
                    json={'security_group_rule': expected_new_rule},
                    validate=dict(json={'security_group_rule': expected_args}),
                ),
            ]
        )
        new_rule = self.cloud.create_security_group_rule(
            secgroup_name_or_id=neutron_grp_dict['id'], **args
        ).to_dict(original_names=True)
        # NOTE(slaweq): don't check location and properties in new rule
        new_rule.pop('created_at')
        new_rule.pop('description')
        new_rule.pop('location')
        new_rule.pop('name')
        new_rule.pop('revision_number')
        new_rule.pop('tags')
        new_rule.pop('updated_at')
        new_rule.pop('if-match')
        self.assertEqual(expected_new_rule, new_rule)
        self.assert_calls()

    def test_create_security_group_rule_nova(self):
        self.has_neutron = False
        self.cloud.secgroup_source = 'nova'

        nova_return = [nova_grp_dict]

        new_rule = fakes.make_fake_nova_security_group_rule(
            id='xyz',
            from_port=1,
            to_port=2000,
            ip_protocol='tcp',
            cidr='1.2.3.4/32',
        )

        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=f'{fakes.COMPUTE_ENDPOINT}/os-security-groups',
                    json={'security_groups': nova_return},
                ),
                dict(
                    method='POST',
                    uri=f'{fakes.COMPUTE_ENDPOINT}/os-security-group-rules',
                    json={'security_group_rule': new_rule},
                    validate=dict(
                        json={
                            "security_group_rule": {
                                "from_port": 1,
                                "ip_protocol": "tcp",
                                "to_port": 2000,
                                "parent_group_id": "2",
                                "cidr": "1.2.3.4/32",
                                "group_id": "123",
                            }
                        }
                    ),
                ),
            ]
        )

        self.cloud.create_security_group_rule(
            '2',
            port_range_min=1,
            port_range_max=2000,
            protocol='tcp',
            remote_ip_prefix='1.2.3.4/32',
            remote_group_id='123',
        )

        self.assert_calls()

    def test_create_security_group_rule_nova_no_ports(self):
        self.has_neutron = False
        self.cloud.secgroup_source = 'nova'

        new_rule = fakes.make_fake_nova_security_group_rule(
            id='xyz',
            from_port=1,
            to_port=65535,
            ip_protocol='tcp',
            cidr='1.2.3.4/32',
        )

        nova_return = [nova_grp_dict]

        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=f'{fakes.COMPUTE_ENDPOINT}/os-security-groups',
                    json={'security_groups': nova_return},
                ),
                dict(
                    method='POST',
                    uri=f'{fakes.COMPUTE_ENDPOINT}/os-security-group-rules',
                    json={'security_group_rule': new_rule},
                    validate=dict(
                        json={
                            "security_group_rule": {
                                "from_port": 1,
                                "ip_protocol": "tcp",
                                "to_port": 65535,
                                "parent_group_id": "2",
                                "cidr": "1.2.3.4/32",
                                "group_id": "123",
                            }
                        }
                    ),
                ),
            ]
        )

        self.cloud.create_security_group_rule(
            '2',
            protocol='tcp',
            remote_ip_prefix='1.2.3.4/32',
            remote_group_id='123',
        )

        self.assert_calls()

    def test_create_security_group_rule_none(self):
        self.has_neutron = False
        self.cloud.secgroup_source = None
        self.assertRaises(
            openstack.cloud.OpenStackCloudUnavailableFeature,
            self.cloud.create_security_group_rule,
            '',
        )

    def test_delete_security_group_rule_neutron(self):
        rule_id = "xyz"
        self.cloud.secgroup_source = 'neutron'
        self.register_uris(
            [
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=[
                            'v2.0',
                            'security-group-rules',
                            f'{rule_id}',
                        ],
                    ),
                    json={},
                )
            ]
        )
        self.assertTrue(self.cloud.delete_security_group_rule(rule_id))
        self.assert_calls()

    def test_delete_security_group_rule_nova(self):
        self.has_neutron = False
        self.cloud.secgroup_source = 'nova'
        self.register_uris(
            [
                dict(
                    method='DELETE',
                    uri=f'{fakes.COMPUTE_ENDPOINT}/os-security-group-rules/xyz',
                ),
            ]
        )
        r = self.cloud.delete_security_group_rule('xyz')
        self.assertTrue(r)
        self.assert_calls()

    def test_delete_security_group_rule_none(self):
        self.has_neutron = False
        self.cloud.secgroup_source = None
        self.assertRaises(
            openstack.cloud.OpenStackCloudUnavailableFeature,
            self.cloud.delete_security_group_rule,
            '',
        )

    def test_delete_security_group_rule_not_found(self):
        rule_id = "doesNotExist"
        self.cloud.secgroup_source = 'neutron'
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'security-groups']
                    ),
                    json={'security_groups': [neutron_grp_dict]},
                )
            ]
        )
        self.assertFalse(self.cloud.delete_security_group(rule_id))
        self.assert_calls()

    def test_delete_security_group_rule_not_found_nova(self):
        self.has_neutron = False
        self.cloud.secgroup_source = 'nova'
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=f'{fakes.COMPUTE_ENDPOINT}/os-security-groups',
                    json={'security_groups': [nova_grp_dict]},
                ),
            ]
        )
        r = self.cloud.delete_security_group('doesNotExist')
        self.assertFalse(r)

        self.assert_calls()

    def test_nova_egress_security_group_rule(self):
        self.has_neutron = False
        self.cloud.secgroup_source = 'nova'
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=f'{fakes.COMPUTE_ENDPOINT}/os-security-groups',
                    json={'security_groups': [nova_grp_dict]},
                ),
            ]
        )
        self.assertRaises(
            exceptions.SDKException,
            self.cloud.create_security_group_rule,
            secgroup_name_or_id='nova-sec-group',
            direction='egress',
        )

        self.assert_calls()

    def test_list_server_security_groups_nova(self):
        self.has_neutron = False

        server = fakes.make_fake_server('1234', 'server-name', 'ACTIVE')

        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', server['id']]
                    ),
                    json=server,
                ),
                dict(
                    method='GET',
                    uri='{endpoint}/servers/{id}/os-security-groups'.format(
                        endpoint=fakes.COMPUTE_ENDPOINT, id=server['id']
                    ),
                    json={'security_groups': [nova_grp_dict]},
                ),
            ]
        )
        groups = self.cloud.list_server_security_groups(server)
        self.assertEqual(
            groups[0]['rules'][0]['ip_range']['cidr'],
            nova_grp_dict['rules'][0]['ip_range']['cidr'],
        )

        self.assert_calls()

    def test_list_server_security_groups_bad_source(self):
        self.has_neutron = False
        self.cloud.secgroup_source = 'invalid'
        server = dict(id='server_id')
        ret = self.cloud.list_server_security_groups(server)
        self.assertEqual([], ret)

    def test_add_security_group_to_server_nova(self):
        self.has_neutron = False
        self.cloud.secgroup_source = 'nova'

        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=f'{fakes.COMPUTE_ENDPOINT}/os-security-groups',
                    json={'security_groups': [nova_grp_dict]},
                ),
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri='{}/servers/{}/action'.format(
                        fakes.COMPUTE_ENDPOINT, '1234'
                    ),
                    validate=dict(
                        json={'addSecurityGroup': {'name': 'nova-sec-group'}}
                    ),
                    status_code=202,
                ),
            ]
        )

        ret = self.cloud.add_server_security_groups(
            dict(id='1234'), 'nova-sec-group'
        )

        self.assertTrue(ret)

        self.assert_calls()

    def test_add_security_group_to_server_neutron(self):
        # fake to get server by name, server-name must match
        fake_server = fakes.make_fake_server('1234', 'server-name', 'ACTIVE')

        # use neutron for secgroup list and return an existing fake
        self.cloud.secgroup_source = 'neutron'

        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=['servers', 'server-name'],
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=['servers', 'detail'],
                        qs_elements=['name=server-name'],
                    ),
                    json={'servers': [fake_server]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'security-groups']
                    ),
                    json={'security_groups': [neutron_grp_dict]},
                ),
                dict(
                    method='POST',
                    uri='{}/servers/{}/action'.format(
                        fakes.COMPUTE_ENDPOINT, '1234'
                    ),
                    validate=dict(
                        json={
                            'addSecurityGroup': {'name': 'neutron-sec-group'}
                        }
                    ),
                    status_code=202,
                ),
            ]
        )

        self.assertTrue(
            self.cloud.add_server_security_groups(
                'server-name', 'neutron-sec-group'
            )
        )
        self.assert_calls()

    def test_remove_security_group_from_server_nova(self):
        self.has_neutron = False
        self.cloud.secgroup_source = 'nova'

        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=f'{fakes.COMPUTE_ENDPOINT}/os-security-groups',
                    json={'security_groups': [nova_grp_dict]},
                ),
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri='{}/servers/{}/action'.format(
                        fakes.COMPUTE_ENDPOINT, '1234'
                    ),
                    validate=dict(
                        json={
                            'removeSecurityGroup': {'name': 'nova-sec-group'}
                        }
                    ),
                ),
            ]
        )

        ret = self.cloud.remove_server_security_groups(
            dict(id='1234'), 'nova-sec-group'
        )
        self.assertTrue(ret)

        self.assert_calls()

    def test_remove_security_group_from_server_neutron(self):
        # fake to get server by name, server-name must match
        fake_server = fakes.make_fake_server('1234', 'server-name', 'ACTIVE')

        # use neutron for secgroup list and return an existing fake
        self.cloud.secgroup_source = 'neutron'

        validate = {'removeSecurityGroup': {'name': 'neutron-sec-group'}}
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=['servers', 'server-name'],
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=['servers', 'detail'],
                        qs_elements=['name=server-name'],
                    ),
                    json={'servers': [fake_server]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'security-groups']
                    ),
                    json={'security_groups': [neutron_grp_dict]},
                ),
                dict(
                    method='POST',
                    uri='{}/servers/{}/action'.format(
                        fakes.COMPUTE_ENDPOINT, '1234'
                    ),
                    validate=dict(json=validate),
                ),
            ]
        )

        self.assertTrue(
            self.cloud.remove_server_security_groups(
                'server-name', 'neutron-sec-group'
            )
        )
        self.assert_calls()

    def test_add_bad_security_group_to_server_nova(self):
        # fake to get server by name, server-name must match
        fake_server = fakes.make_fake_server('1234', 'server-name', 'ACTIVE')

        # use nova for secgroup list and return an existing fake
        self.has_neutron = False
        self.cloud.secgroup_source = 'nova'
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=['servers', 'server-name'],
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=['servers', 'detail'],
                        qs_elements=['name=server-name'],
                    ),
                    json={'servers': [fake_server]},
                ),
                dict(
                    method='GET',
                    uri=f'{fakes.COMPUTE_ENDPOINT}/os-security-groups',
                    json={'security_groups': [nova_grp_dict]},
                ),
            ]
        )

        ret = self.cloud.add_server_security_groups(
            'server-name', 'unknown-sec-group'
        )
        self.assertFalse(ret)

        self.assert_calls()

    def test_add_bad_security_group_to_server_neutron(self):
        # fake to get server by name, server-name must match
        fake_server = fakes.make_fake_server('1234', 'server-name', 'ACTIVE')

        # use neutron for secgroup list and return an existing fake
        self.cloud.secgroup_source = 'neutron'

        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=['servers', 'server-name'],
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=['servers', 'detail'],
                        qs_elements=['name=server-name'],
                    ),
                    json={'servers': [fake_server]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'security-groups']
                    ),
                    json={'security_groups': [neutron_grp_dict]},
                ),
            ]
        )
        self.assertFalse(
            self.cloud.add_server_security_groups(
                'server-name', 'unknown-sec-group'
            )
        )
        self.assert_calls()

    def test_add_security_group_to_bad_server(self):
        # fake to get server by name, server-name must match
        fake_server = fakes.make_fake_server('1234', 'server-name', 'ACTIVE')

        print(
            self.get_mock_url(
                'compute',
                'public',
                append=['servers', 'unknown-server-name'],
                base_url_append='v2.1',
            )
        )

        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=['servers', 'unknown-server-name'],
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=['servers', 'detail'],
                        qs_elements=['name=unknown-server-name'],
                    ),
                    json={'servers': [fake_server]},
                ),
            ]
        )

        ret = self.cloud.add_server_security_groups(
            'unknown-server-name', 'nova-sec-group'
        )
        self.assertFalse(ret)

        self.assert_calls()

    def test_get_security_group_by_id_neutron(self):
        self.cloud.secgroup_source = 'neutron'
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=[
                            'v2.0',
                            'security-groups',
                            neutron_grp_dict['id'],
                        ],
                    ),
                    json={'security_group': neutron_grp_dict},
                )
            ]
        )
        ret_sg = self.cloud.get_security_group_by_id(neutron_grp_dict['id'])
        self.assertEqual(neutron_grp_dict['id'], ret_sg['id'])
        self.assertEqual(neutron_grp_dict['name'], ret_sg['name'])
        self.assertEqual(
            neutron_grp_dict['description'], ret_sg['description']
        )
        self.assertEqual(neutron_grp_dict['stateful'], ret_sg['stateful'])
        self.assert_calls()

    def test_get_security_group_by_id_nova(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri='{endpoint}/os-security-groups/{id}'.format(
                        endpoint=fakes.COMPUTE_ENDPOINT, id=nova_grp_dict['id']
                    ),
                    json={'security_group': nova_grp_dict},
                ),
            ]
        )
        self.cloud.secgroup_source = 'nova'
        self.has_neutron = False
        ret_sg = self.cloud.get_security_group_by_id(nova_grp_dict['id'])
        self.assertEqual(nova_grp_dict['id'], ret_sg['id'])
        self.assertEqual(nova_grp_dict['name'], ret_sg['name'])
        self.assert_calls()

    def test_normalize_secgroups(self):
        nova_secgroup = dict(
            id='abc123',
            name='nova_secgroup',
            description='A Nova security group',
            rules=[
                dict(
                    id='123',
                    from_port=80,
                    to_port=81,
                    ip_protocol='tcp',
                    ip_range={'cidr': '0.0.0.0/0'},
                    parent_group_id='xyz123',
                )
            ],
        )
        expected = dict(
            id='abc123',
            name='nova_secgroup',
            description='A Nova security group',
            project_id='',
            tenant_id='',
            properties={},
            location=dict(
                region_name='RegionOne',
                zone=None,
                project=dict(
                    domain_name='default',
                    id='1c36b64c840a42cd9e9b931a369337f0',
                    domain_id=None,
                    name='admin',
                ),
                cloud='_test_cloud_',
            ),
            security_group_rules=[
                dict(
                    id='123',
                    direction='ingress',
                    ethertype='IPv4',
                    port_range_min=80,
                    port_range_max=81,
                    protocol='tcp',
                    remote_ip_prefix='0.0.0.0/0',
                    security_group_id='xyz123',
                    project_id='',
                    tenant_id='',
                    properties={},
                    remote_group_id=None,
                    location=dict(
                        region_name='RegionOne',
                        zone=None,
                        project=dict(
                            domain_name='default',
                            id='1c36b64c840a42cd9e9b931a369337f0',
                            domain_id=None,
                            name='admin',
                        ),
                        cloud='_test_cloud_',
                    ),
                )
            ],
        )
        # Set secgroup source to nova for this test as stateful parameter
        # is only valid for neutron security groups.
        self.cloud.secgroup_source = 'nova'
        retval = self.cloud._normalize_secgroup(nova_secgroup)
        self.cloud.secgroup_source = 'neutron'
        self.assertEqual(expected, retval)

    def test_normalize_secgroups_negone_port(self):
        nova_secgroup = dict(
            id='abc123',
            name='nova_secgroup',
            description='A Nova security group with -1 ports',
            rules=[
                dict(
                    id='123',
                    from_port=-1,
                    to_port=-1,
                    ip_protocol='icmp',
                    ip_range={'cidr': '0.0.0.0/0'},
                    parent_group_id='xyz123',
                )
            ],
        )
        retval = self.cloud._normalize_secgroup(nova_secgroup)
        self.assertIsNone(retval['security_group_rules'][0]['port_range_min'])
        self.assertIsNone(retval['security_group_rules'][0]['port_range_max'])

    def test_normalize_secgroup_rules(self):
        nova_rules = [
            dict(
                id='123',
                from_port=80,
                to_port=81,
                ip_protocol='tcp',
                ip_range={'cidr': '0.0.0.0/0'},
                parent_group_id='xyz123',
            )
        ]
        expected = [
            dict(
                id='123',
                direction='ingress',
                ethertype='IPv4',
                port_range_min=80,
                port_range_max=81,
                protocol='tcp',
                remote_ip_prefix='0.0.0.0/0',
                security_group_id='xyz123',
                tenant_id='',
                project_id='',
                remote_group_id=None,
                properties={},
                location=dict(
                    region_name='RegionOne',
                    zone=None,
                    project=dict(
                        domain_name='default',
                        id='1c36b64c840a42cd9e9b931a369337f0',
                        domain_id=None,
                        name='admin',
                    ),
                    cloud='_test_cloud_',
                ),
            )
        ]
        retval = self.cloud._normalize_secgroup_rules(nova_rules)
        self.assertEqual(expected, retval)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.image.v2 import _proxy as _image_v2
from openstack.tests.functional import base
from openstack import utils


class BaseImageTest(base.BaseFunctionalTest):
    _wait_for_timeout_key = 'OPENSTACKSDK_FUNC_TEST_TIMEOUT_IMAGE'

    admin_image_client: _image_v2.Proxy
    image_client: _image_v2.Proxy

    def setUp(self):
        super().setUp()
        self._set_user_cloud(image_api_version='2')
        self._set_operator_cloud(image_api_version='2')

        if not self.user_cloud.has_service('image', '2'):
            self.skipTest('image service not supported by cloud')
        self.admin_image_client = utils.ensure_service_version(
            self.operator_cloud.image, '2'
        )
        self.image_client = utils.ensure_service_version(
            self.user_cloud.image, '2'
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock
import warnings

from openstack.block_storage.v2 import _proxy
from openstack.block_storage.v2 import backup
from openstack.block_storage.v2 import capabilities
from openstack.block_storage.v2 import limits
from openstack.block_storage.v2 import quota_class_set
from openstack.block_storage.v2 import quota_set
from openstack.block_storage.v2 import service
from openstack.block_storage.v2 import snapshot
from openstack.block_storage.v2 import stats
from openstack.block_storage.v2 import type
from openstack.block_storage.v2 import volume
from openstack.identity.v3 import project
from openstack import proxy as proxy_base
from openstack.tests.unit import test_proxy_base
from openstack import warnings as os_warnings


class TestVolumeProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super().setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestVolume(TestVolumeProxy):
    def test_volume_get(self):
        self.verify_get(self.proxy.get_volume, volume.Volume)

    def test_volume_find(self):
        self.verify_find(
            self.proxy.find_volume,
            volume.Volume,
            method_kwargs={'all_projects': True},
            expected_kwargs={
                'list_base_path': '/volumes/detail',
                'all_projects': True,
            },
        )

    def test_volumes_detailed(self):
        self.verify_list(
            self.proxy.volumes,
            volume.Volume,
            method_kwargs={"details": True, "all_projects": True},
            expected_kwargs={
                "base_path": "/volumes/detail",
                "all_projects": True,
            },
        )

    def test_volumes_not_detailed(self):
        self.verify_list(
            self.proxy.volumes,
            volume.Volume,
            method_kwargs={"details": False, "all_projects": True},
            expected_kwargs={"all_projects": True},
        )

    def test_volume_create_attrs(self):
        self.verify_create(self.proxy.create_volume, volume.Volume)

    def test_volume_delete(self):
        self._verify(
            "openstack.block_storage.v2.volume.Volume.delete",
            self.proxy.delete_volume,
            method_args=["value"],
            expected_args=[self.proxy],
            expected_kwargs={"params": {"cascade": False}},
        )

    def test_volume_delete_force(self):
        self._verify(
            "openstack.block_storage.v2.volume.Volume.force_delete",
            self.proxy.delete_volume,
            method_args=["value"],
            method_kwargs={"force": True},
            expected_args=[self.proxy],
        )

    def test_get_volume_metadata(self):
        self._verify(
            "openstack.block_storage.v2.volume.Volume.fetch_metadata",
            self.proxy.get_volume_metadata,
            method_args=["value"],
            expected_args=[self.proxy],
            expected_result=volume.Volume(id="value", metadata={}),
        )

    def test_set_volume_metadata(self):
        kwargs = {"a": "1", "b": "2"}
        id = "an_id"
        self._verify(
            "openstack.block_storage.v2.volume.Volume.set_metadata",
            self.proxy.set_volume_metadata,
            method_args=[id],
            method_kwargs=kwargs,
            method_result=volume.Volume.existing(id=id, metadata=kwargs),
            expected_args=[self.proxy],
            expected_kwargs={'metadata': kwargs},
            expected_result=volume.Volume.existing(id=id, metadata=kwargs),
        )

    def test_delete_volume_metadata(self):
        self._verify(
            "openstack.block_storage.v2.volume.Volume.delete_metadata_item",
            self.proxy.delete_volume_metadata,
            expected_result=None,
            method_args=["value", ["key"]],
            expected_args=[self.proxy, "key"],
        )

    def test_backend_pools(self):
        self.verify_list(self.proxy.backend_pools, stats.Pools)

    def test_volume_wait_for(self):
        value = volume.Volume(id='1234')
        self.verify_wait_for_status(
            self.proxy.wait_for_status,
            method_args=[value],
            expected_args=[
                self.proxy,
                value,
                'available',
                ['error'],
                2,
                None,
                'status',
                None,
            ],
        )


class TestVolumeActions(TestVolumeProxy):
    def test_volume_extend(self):
        self._verify(
            "openstack.block_storage.v2.volume.Volume.extend",
            self.proxy.extend_volume,
            method_args=["value", "new-size"],
            expected_args=[self.proxy, "new-size"],
        )

    def test_volume_set_readonly_no_argument(self):
        self._verify(
            "openstack.block_storage.v2.volume.Volume.set_readonly",
            self.proxy.set_volume_readonly,
            method_args=["value"],
            expected_args=[self.proxy, True],
        )

    def test_volume_set_readonly_false(self):
        self._verify(
            "openstack.block_storage.v2.volume.Volume.set_readonly",
            self.proxy.set_volume_readonly,
            method_args=["value", False],
            expected_args=[self.proxy, False],
        )

    def test_volume_set_bootable(self):
        self._verify(
            "openstack.block_storage.v2.volume.Volume.set_bootable_status",
            self.proxy.set_volume_bootable_status,
            method_args=["value", True],
            expected_args=[self.proxy, True],
        )

    def test_volume_reset_status(self):
        self._verify(
            "openstack.block_storage.v2.volume.Volume.reset_status",
            self.proxy.reset_volume_status,
            method_args=["value", '1', '2', '3'],
            expected_args=[self.proxy, '1', '2', '3'],
        )

    def test_set_volume_image_metadata(self):
        self._verify(
            "openstack.block_storage.v2.volume.Volume.set_image_metadata",
            self.proxy.set_volume_image_metadata,
            method_args=["value"],
            method_kwargs={'foo': 'bar'},
            expected_args=[self.proxy],
            expected_kwargs={'metadata': {'foo': 'bar'}},
        )

    def test_delete_volume_image_metadata(self):
        self._verify(
            "openstack.block_storage.v2.volume.Volume.delete_image_metadata",
            self.proxy.delete_volume_image_metadata,
            method_args=["value"],
            expected_args=[self.proxy],
        )

    def test_delete_volume_image_metadata__with_keys(self):
        self._verify(
            "openstack.block_storage.v2.volume.Volume.delete_image_metadata_item",
            self.proxy.delete_volume_image_metadata,
            method_args=["value", ['foo']],
            expected_args=[self.proxy, 'foo'],
        )

    def test_attach_instance(self):
        self._verify(
            "openstack.block_storage.v2.volume.Volume.attach",
            self.proxy.attach_volume,
            method_args=["value", '1'],
            method_kwargs={'instance': '2'},
            expected_args=[self.proxy, '1', '2', None],
        )

    def test_attach_host(self):
        self._verify(
            "openstack.block_storage.v2.volume.Volume.attach",
            self.proxy.attach_volume,
            method_args=["value", '1'],
            method_kwargs={'host_name': '3'},
            expected_args=[self.proxy, '1', None, '3'],
        )

    def test_detach_defaults(self):
        self._verify(
            "openstack.block_storage.v2.volume.Volume.detach",
            self.proxy.detach_volume,
            method_args=["value", '1'],
            expected_args=[self.proxy, '1', False, None],
        )

    def test_detach_force(self):
        self._verify(
            "openstack.block_storage.v2.volume.Volume.detach",
            self.proxy.detach_volume,
            method_args=["value", '1', True, {'a': 'b'}],
            expected_args=[self.proxy, '1', True, {'a': 'b'}],
        )

    def test_unmanage(self):
        self._verify(
            "openstack.block_storage.v2.volume.Volume.unmanage",
            self.proxy.unmanage_volume,
            method_args=["value"],
            expected_args=[self.proxy],
        )

    def test_migrate_default(self):
        self._verify(
            "openstack.block_storage.v2.volume.Volume.migrate",
            self.proxy.migrate_volume,
            method_args=["value", '1'],
            expected_args=[self.proxy, '1', False, False],
        )

    def test_migrate_nondefault(self):
        self._verify(
            "openstack.block_storage.v2.volume.Volume.migrate",
            self.proxy.migrate_volume,
            method_args=["value", '1', True, True],
            expected_args=[self.proxy, '1', True, True],
        )

    def test_complete_migration(self):
        self._verify(
            "openstack.block_storage.v2.volume.Volume.complete_migration",
            self.proxy.complete_volume_migration,
            method_args=["value", '1'],
            expected_args=[self.proxy, "1", False],
        )

    def test_complete_migration_error(self):
        self._verify(
            "openstack.block_storage.v2.volume.Volume.complete_migration",
            self.proxy.complete_volume_migration,
            method_args=["value", "1", True],
            expected_args=[self.proxy, "1", True],
        )


class TestBackup(TestVolumeProxy):
    def test_backups_detailed(self):
        self.verify_list(
            self.proxy.backups,
            backup.Backup,
            method_kwargs={"details": True, "query": 1},
            expected_kwargs={"query": 1, "base_path": "/backups/detail"},
        )

    def test_backups_not_detailed(self):
        self.verify_list(
            self.proxy.backups,
            backup.Backup,
            method_kwargs={"details": False, "query": 1},
            expected_kwargs={"query": 1},
        )

    def test_backup_get(self):
        self.verify_get(self.proxy.get_backup, backup.Backup)

    def test_backup_find(self):
        self.verify_find(
            self.proxy.find_backup,
            backup.Backup,
            expected_kwargs={'list_base_path': '/backups/detail'},
        )

    def test_backup_delete(self):
        self.verify_delete(self.proxy.delete_backup, backup.Backup, False)

    def test_backup_delete_ignore(self):
        self.verify_delete(self.proxy.delete_backup, backup.Backup, True)

    def test_backup_delete_force(self):
        self._verify(
            "openstack.block_storage.v2.backup.Backup.force_delete",
            self.proxy.delete_backup,
            method_args=["value"],
            method_kwargs={"force": True},
            expected_args=[self.proxy],
        )

    def test_backup_create_attrs(self):
        self.verify_create(self.proxy.create_backup, backup.Backup)

    def test_backup_restore(self):
        self._verify(
            'openstack.block_storage.v2.backup.Backup.restore',
            self.proxy.restore_backup,
            method_args=['volume_id'],
            method_kwargs={'volume_id': 'vol_id', 'name': 'name'},
            expected_args=[self.proxy],
            expected_kwargs={'volume_id': 'vol_id', 'name': 'name'},
        )

    def test_backup_reset_status(self):
        self._verify(
            "openstack.block_storage.v2.backup.Backup.reset_status",
            self.proxy.reset_backup_status,
            method_args=["value", "new_status"],
            expected_args=[self.proxy, "new_status"],
        )


class TestLimit(TestVolumeProxy):
    def test_limits_get(self):
        self.verify_get(
            self.proxy.get_limits,
            limits.Limits,
            method_args=[],
            expected_kwargs={'requires_id': False},
        )


class TestCapabilities(TestVolumeProxy):
    def test_capabilites_get(self):
        self.verify_get(self.proxy.get_capabilities, capabilities.Capabilities)


class TestSnapshot(TestVolumeProxy):
    def test_snapshot_get(self):
        self.verify_get(self.proxy.get_snapshot, snapshot.Snapshot)

    def test_snapshot_find(self):
        self.verify_find(
            self.proxy.find_snapshot,
            snapshot.Snapshot,
            method_kwargs={'all_projects': True},
            expected_kwargs={
                'list_base_path': '/snapshots/detail',
                'all_projects': True,
            },
        )

    def test_snapshots_detailed(self):
        self.verify_list(
            self.proxy.snapshots,
            snapshot.SnapshotDetail,
            method_kwargs={"details": True, "all_projects": True},
            expected_kwargs={"all_projects": True},
        )

    def test_snapshots_not_detailed(self):
        self.verify_list(
            self.proxy.snapshots,
            snapshot.Snapshot,
            method_kwargs={"details": False, "all_projects": True},
            expected_kwargs={"all_projects": 1},
        )

    def test_snapshot_create_attrs(self):
        self.verify_create(self.proxy.create_snapshot, snapshot.Snapshot)

    def test_snapshot_update(self):
        self.verify_update(self.proxy.update_snapshot, snapshot.Snapshot)

    def test_snapshot_delete(self):
        self.verify_delete(
            self.proxy.delete_snapshot, snapshot.Snapshot, False
        )

    def test_snapshot_delete_ignore(self):
        self.verify_delete(self.proxy.delete_snapshot, snapshot.Snapshot, True)

    def test_snapshot_reset_status(self):
        self._verify(
            "openstack.block_storage.v2.snapshot.Snapshot.reset_status",
            self.proxy.reset_snapshot_status,
            method_args=["value", "new_status"],
            expected_args=[self.proxy, "new_status"],
        )

    def test_snapshot_manage(self):
        kwargs = {
            "volume_id": "fake_id",
            "remote_source": "fake_volume",
            "snapshot_name": "fake_snap",
            "description": "test_snap",
            "property": {"k": "v"},
        }
        self._verify(
            "openstack.block_storage.v2.snapshot.Snapshot.manage",
            self.proxy.manage_snapshot,
            method_kwargs=kwargs,
            method_result=snapshot.Snapshot(id="fake_id"),
            expected_args=[self.proxy],
            expected_kwargs=kwargs,
            expected_result=snapshot.Snapshot(id="fake_id"),
        )

    def test_snapshot_unmanage(self):
        self._verify(
            "openstack.block_storage.v2.snapshot.Snapshot.unmanage",
            self.proxy.unmanage_snapshot,
            method_args=["value"],
            expected_args=[self.proxy],
            expected_result=None,
        )

    def test_get_snapshot_metadata(self):
        self._verify(
            "openstack.block_storage.v2.snapshot.Snapshot.fetch_metadata",
            self.proxy.get_snapshot_metadata,
            method_args=["value"],
            expected_args=[self.proxy],
            expected_result=snapshot.Snapshot(id="value", metadata={}),
        )

    def test_set_snapshot_metadata(self):
        kwargs = {"a": "1", "b": "2"}
        id = "an_id"
        self._verify(
            "openstack.block_storage.v2.snapshot.Snapshot.set_metadata",
            self.proxy.set_snapshot_metadata,
            method_args=[id],
            method_kwargs=kwargs,
            method_result=snapshot.Snapshot.existing(id=id, metadata=kwargs),
            expected_args=[self.proxy],
            expected_kwargs={'metadata': kwargs},
            expected_result=snapshot.Snapshot.existing(id=id, metadata=kwargs),
        )

    def test_delete_snapshot_metadata(self):
        self._verify(
            "openstack.block_storage.v2.snapshot.Snapshot."
            "delete_metadata_item",
            self.proxy.delete_snapshot_metadata,
            expected_result=None,
            method_args=["value", ["key"]],
            expected_args=[self.proxy, "key"],
        )


class TestType(TestVolumeProxy):
    def test_type_get(self):
        self.verify_get(self.proxy.get_type, type.Type)

    def test_type_find(self):
        self.verify_find(self.proxy.find_type, type.Type)

    def test_types(self):
        self.verify_list(self.proxy.types, type.Type)

    def test_type_create_attrs(self):
        self.verify_create(self.proxy.create_type, type.Type)

    def test_type_delete(self):
        self.verify_delete(self.proxy.delete_type, type.Type, False)

    def test_type_delete_ignore(self):
        self.verify_delete(self.proxy.delete_type, type.Type, True)

    def test_type_get_private_access(self):
        self._verify(
            "openstack.block_storage.v2.type.Type.get_private_access",
            self.proxy.get_type_access,
            method_args=["value"],
            expected_args=[self.proxy],
        )

    def test_type_add_private_access(self):
        self._verify(
            "openstack.block_storage.v2.type.Type.add_private_access",
            self.proxy.add_type_access,
            method_args=["value", "a"],
            expected_args=[self.proxy, "a"],
        )

    def test_type_remove_private_access(self):
        self._verify(
            "openstack.block_storage.v2.type.Type.remove_private_access",
            self.proxy.remove_type_access,
            method_args=["value", "a"],
            expected_args=[self.proxy, "a"],
        )


class TestQuotaClassSet(TestVolumeProxy):
    def test_quota_class_set_get(self):
        self.verify_get(
            self.proxy.get_quota_class_set, quota_class_set.QuotaClassSet
        )

    def test_quota_class_set_update(self):
        self.verify_update(
            self.proxy.update_quota_class_set,
            quota_class_set.QuotaClassSet,
            False,
        )


class TestQuotaSet(TestVolumeProxy):
    def test_quota_set_get(self):
        self._verify(
            'openstack.resource.Resource.fetch',
            self.proxy.get_quota_set,
            method_args=['prj'],
            expected_args=[
                self.proxy,
                False,
                None,
                None,
                False,
            ],
            expected_kwargs={
                'microversion': None,
                'resource_response_key': None,
                'usage': False,
            },
            method_result=quota_set.QuotaSet(),
            expected_result=quota_set.QuotaSet(),
        )

    def test_quota_set_get_query(self):
        self._verify(
            'openstack.resource.Resource.fetch',
            self.proxy.get_quota_set,
            method_args=['prj'],
            method_kwargs={'usage': True, 'user_id': 'uid'},
            expected_args=[
                self.proxy,
                False,
                None,
                None,
                False,
            ],
            expected_kwargs={
                'microversion': None,
                'resource_response_key': None,
                'usage': True,
                'user_id': 'uid',
            },
        )

    def test_quota_set_get_defaults(self):
        self._verify(
            'openstack.resource.Resource.fetch',
            self.proxy.get_quota_set_defaults,
            method_args=['prj'],
            expected_args=[
                self.proxy,
                False,
                '/os-quota-sets/defaults',
                None,
                False,
            ],
            expected_kwargs={
                'microversion': None,
                'resource_response_key': None,
            },
        )

    def test_quota_set_reset(self):
        self._verify(
            'openstack.resource.Resource.delete',
            self.proxy.revert_quota_set,
            method_args=['prj'],
            method_kwargs={'user_id': 'uid'},
            expected_args=[self.proxy],
            expected_kwargs={'user_id': 'uid'},
        )

    @mock.patch.object(proxy_base.Proxy, '_get_resource')
    def test_quota_set_update(self, mock_get):
        fake_project = project.Project(id='prj')
        mock_get.side_effect = [fake_project]

        self._verify(
            'openstack.proxy.Proxy._update',
            self.proxy.update_quota_set,
            method_args=['prj'],
            method_kwargs={'volumes': 123},
            expected_args=[quota_set.QuotaSet, None],
            expected_kwargs={'project_id': 'prj', 'volumes': 123},
        )
        mock_get.assert_called_once_with(project.Project, 'prj')

    @mock.patch.object(proxy_base.Proxy, "_get_resource")
    def test_quota_set_update__legacy(self, mock_get):
        fake_quota_set = quota_set.QuotaSet(project_id='prj')
        mock_get.side_effect = [fake_quota_set]
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter('always')

            self._verify(
                'openstack.resource.Resource.commit',
                self.proxy.update_quota_set,
                method_args=[fake_quota_set],
                method_kwargs={'ram': 123},
                expected_args=[self.proxy],
                expected_kwargs={},
            )

            self.assertEqual(1, len(w))
            self.assertEqual(
                os_warnings.RemovedInSDK50Warning,
                w[-1].category,
            )
            self.assertIn(
                "The signature of 'update_quota_set' has changed ",
                str(w[-1]),
            )


class TestService(TestVolumeProxy):
    def test_services(self):
        self.verify_list(self.proxy.services, service.Service)

    def test_enable_service(self):
        self._verify(
            'openstack.block_storage.v2.service.Service.enable',
            self.proxy.enable_service,
            method_args=["value"],
            expected_args=[self.proxy],
        )

    def test_disable_service(self):
        self._verify(
            'openstack.block_storage.v2.service.Service.disable',
            self.proxy.disable_service,
            method_args=["value"],
            expected_kwargs={"reason": None},
            expected_args=[self.proxy],
        )

    def test_thaw_service(self):
        self._verify(
            'openstack.block_storage.v2.service.Service.thaw',
            self.proxy.thaw_service,
            method_args=["value"],
            expected_args=[self.proxy],
        )

    def test_freeze_service(self):
        self._verify(
            'openstack.block_storage.v2.service.Service.freeze',
            self.proxy.freeze_service,
            method_args=["value"],
            expected_args=[self.proxy],
        )

    def test_failover_service(self):
        self._verify(
            'openstack.block_storage.v2.service.Service.failover',
            self.proxy.failover_service,
            method_args=["value"],
            expected_args=[self.proxy],
            expected_kwargs={"backend_id": None},
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.identity.v3 import mapping
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'id': IDENTIFIER,
    'rules': [{'local': [], 'remote': []}],
    'schema_version': '2.0',
}


class TestMapping(base.TestCase):
    def test_basic(self):
        sot = mapping.Mapping()
        self.assertEqual('mapping', sot.resource_key)
        self.assertEqual('mappings', sot.resources_key)
        self.assertEqual('/OS-FEDERATION/mappings', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertEqual('PATCH', sot.commit_method)
        self.assertEqual('PUT', sot.create_method)

        self.assertDictEqual(
            {
                'limit': 'limit',
                'marker': 'marker',
            },
            sot._query_mapping._mapping,
        )

    def test_make_it(self):
        sot = mapping.Mapping(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['rules'], sot.rules)
        self.assertEqual(EXAMPLE['schema_version'], sot.schema_version)



# Copyright (c) 2018 China Telecom Corporation
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from openstack.network.v2 import firewall_group
from openstack.tests.functional import base


class TestFirewallGroup(base.BaseFunctionalTest):
    ID: str

    def setUp(self):
        super().setUp()
        if not self.user_cloud._has_neutron_extension("fwaas_v2"):
            self.skipTest("fwaas_v2 service not supported by cloud")
        self.NAME = self.getUniqueString()
        sot = self.user_cloud.network.create_firewall_group(name=self.NAME)
        assert isinstance(sot, firewall_group.FirewallGroup)
        self.assertEqual(self.NAME, sot.name)
        self.ID = sot.id

    def tearDown(self):
        sot = self.user_cloud.network.delete_firewall_group(
            self.ID, ignore_missing=False
        )
        self.assertIs(None, sot)
        super().tearDown()

    def test_find(self):
        sot = self.user_cloud.network.find_firewall_group(
            self.NAME, ignore_missing=False
        )
        self.assertEqual(self.ID, sot.id)

    def test_get(self):
        sot = self.user_cloud.network.get_firewall_group(self.ID)
        self.assertEqual(self.NAME, sot.name)
        self.assertEqual(self.ID, sot.id)

    def test_list(self):
        names = [o.name for o in self.user_cloud.network.firewall_groups()]
        self.assertIn(self.NAME, names)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.block_storage.v3 import volume as _volume
from openstack.compute.v2 import server as _server
from openstack.compute.v2 import volume_attachment as _volume_attachment
from openstack.tests.functional.compute.v2 import base


class TestServerVolumeAttachment(base.BaseComputeTest):
    def setUp(self):
        super().setUp()

        self.server_name = self.getUniqueString()
        self.volume_name = self.getUniqueString()

        # create the server and volume

        server = self.compute_client.create_server(
            name=self.server_name,
            flavor_id=self.flavor.id,
            image_id=self.image.id,
            networks='none',
        )
        self.compute_client.wait_for_server(
            server, wait=self._wait_for_timeout
        )
        self.addCleanup(self._delete_server, server)
        self.assertIsInstance(server, _server.Server)
        self.assertEqual(self.server_name, server.name)

        volume = self.block_storage_client.create_volume(
            name=self.volume_name, size=1
        )
        self.block_storage_client.wait_for_status(
            volume, status='available', wait=self._wait_for_timeout
        )
        self.addCleanup(self._delete_volume, volume)
        self.assertIsInstance(volume, _volume.Volume)
        self.assertEqual(self.volume_name, volume.name)

        self.server = server
        self.volume = volume

    def _delete_server(self, server):
        self.compute_client.delete_server(server.id)
        self.compute_client.wait_for_delete(
            server, wait=self._wait_for_timeout
        )

    def _delete_volume(self, volume):
        self.block_storage_client.delete_volume(volume.id)
        self.block_storage_client.wait_for_delete(
            volume, wait=self._wait_for_timeout
        )

    def test_volume_attachment(self):
        # create the volume attachment

        volume_attachment = self.compute_client.create_volume_attachment(
            self.server, self.volume
        )
        self.assertIsInstance(
            volume_attachment, _volume_attachment.VolumeAttachment
        )
        self.block_storage_client.wait_for_status(
            self.volume, status='in-use', wait=self._wait_for_timeout
        )

        # list all attached volume attachments (there should only be one)

        volume_attachments = list(
            self.compute_client.volume_attachments(self.server)
        )
        self.assertEqual(1, len(volume_attachments))
        self.assertIsInstance(
            volume_attachments[0], _volume_attachment.VolumeAttachment
        )

        # update the volume attachment

        volume_attachment = self.compute_client.update_volume_attachment(
            self.server, self.volume, delete_on_termination=True
        )
        self.assertIsInstance(
            volume_attachment, _volume_attachment.VolumeAttachment
        )

        # retrieve details of the (updated) volume attachment

        volume_attachment = self.compute_client.get_volume_attachment(
            self.server, self.volume
        )
        self.assertIsInstance(
            volume_attachment, _volume_attachment.VolumeAttachment
        )
        self.assertTrue(volume_attachment.delete_on_termination)

        # delete the volume attachment

        result = self.compute_client.delete_volume_attachment(
            self.server, self.volume, ignore_missing=False
        )
        self.assertIsNone(result)

        self.block_storage_client.wait_for_status(
            self.volume, status='available', wait=self._wait_for_timeout
        )

        # Wait for the attachment to be deleted.
        # This is done to prevent a race between the BDM
        # record being deleted and we trying to delete the server.
        self.compute_client.wait_for_delete(
            volume_attachment, wait=self._wait_for_timeout
        )

        # Verify the server doesn't have any volume attachment
        volume_attachments = list(
            self.compute_client.volume_attachments(self.server)
        )
        self.assertEqual(0, len(volume_attachments))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
# mypy: disable-error-code="method-assign"

from openstack.network.v2 import router
from openstack.tests.functional.network.v2 import common


class TestRouter(common.TestTagNeutron):
    def setUp(self):
        super().setUp()
        self.NAME = self.getUniqueString()
        self.UPDATE_NAME = self.getUniqueString()
        sot = self.user_cloud.network.create_router(name=self.NAME)
        assert isinstance(sot, router.Router)
        self.assertEqual(self.NAME, sot.name)
        self.ID = sot.id
        self.get_command = self.user_cloud.network.get_router

    def tearDown(self):
        sot = self.user_cloud.network.delete_router(
            self.ID, ignore_missing=False
        )
        self.assertIsNone(sot)
        super().tearDown()

    def test_find(self):
        sot = self.user_cloud.network.find_router(
            self.NAME, ignore_missing=False
        )
        self.assertEqual(self.ID, sot.id)

    def test_get(self):
        sot = self.user_cloud.network.get_router(self.ID)
        self.assertEqual(self.NAME, sot.name)
        self.assertEqual(self.ID, sot.id)
        if not self.user_cloud._has_neutron_extension("l3-ha"):
            self.assertFalse(sot.is_ha)

    def test_list(self):
        names = [o.name for o in self.user_cloud.network.routers()]
        self.assertIn(self.NAME, names)
        if not self.user_cloud._has_neutron_extension("l3-ha"):
            ha = [o.is_ha for o in self.user_cloud.network.routers()]
            self.assertIn(False, ha)

    def test_update(self):
        sot = self.user_cloud.network.update_router(
            self.ID, name=self.UPDATE_NAME
        )
        self.assertEqual(self.UPDATE_NAME, sot.name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import address_group
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'id': IDENTIFIER,
    'name': '1',
    'description': '2',
    'project_id': '3',
    'addresses': ['10.0.0.1/32'],
}


class TestAddressGroup(base.TestCase):
    def test_basic(self):
        sot = address_group.AddressGroup()
        self.assertEqual('address_group', sot.resource_key)
        self.assertEqual('address_groups', sot.resources_key)
        self.assertEqual('/address-groups', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

        self.assertDictEqual(
            {
                "name": "name",
                "description": "description",
                "project_id": "project_id",
                "sort_key": "sort_key",
                "sort_dir": "sort_dir",
                "limit": "limit",
                "marker": "marker",
            },
            sot._query_mapping._mapping,
        )

    def test_make_it(self):
        sot = address_group.AddressGroup(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertCountEqual(EXAMPLE['addresses'], sot.addresses)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class RoleProjectGroupAssignment(resource.Resource):
    resource_key = 'role'
    resources_key = 'roles'
    base_path = '/projects/%(project_id)s/groups/%(group_id)s/roles'

    # capabilities
    allow_list = True

    # Properties
    #: name of the role *Type: string*
    name = resource.Body('name')
    #: The links for the service resource.
    links = resource.Body('links')
    #: The ID of the project to list assignment from. *Type: string*
    project_id = resource.URI('project_id')
    #: The ID of the group to list assignment from. *Type: string*
    group_id = resource.URI('group_id')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class ServiceProvider(resource.Resource):
    resource_key = 'service_provider'
    resources_key = 'service_providers'
    base_path = '/OS-FEDERATION/service_providers'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True
    create_method = 'PUT'
    create_exclude_id_from_body = True
    commit_method = 'PATCH'

    _query_mapping = resource.QueryParameters(
        'id',
        is_enabled='enabled',
    )

    # Properties
    #: The URL to authenticate against.
    auth_url = resource.Body('auth_url')
    #: A description of this service provider.
    description = resource.Body('description')
    #: If the service provider is currently enabled.
    is_enabled = resource.Body('enabled', type=bool)
    #: The identifier of the service provider.
    name = resource.Body('id')
    #: The prefix of the RelayState SAML attribute.
    relay_state_prefix = resource.Body('relay_state_prefix')
    #: The service provider's URL.
    sp_url = resource.Body('sp_url')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_users
----------------------------------

Functional tests for user methods.
"""

from openstack import exceptions
from openstack.tests.functional import base


class TestUsers(base.KeystoneBaseFunctionalTest):
    def setUp(self):
        super().setUp()
        if not self.operator_cloud:
            self.skipTest("Operator cloud is required for this test")

        self.user_prefix = self.getUniqueString('user')
        self.addCleanup(self._cleanup_users)

    def _cleanup_users(self):
        exception_list = list()
        for user in self.operator_cloud.list_users():
            if user['name'].startswith(self.user_prefix):
                try:
                    self.operator_cloud.delete_user(user['id'])
                except Exception as e:
                    exception_list.append(str(e))
                    continue

        if exception_list:
            raise exceptions.SDKException('\n'.join(exception_list))

    def _create_user(self, **kwargs):
        domain = self.operator_cloud.get_domain('default')
        return self.operator_cloud.create_user(
            domain_id=domain['id'], **kwargs
        )

    def test_list_users(self):
        users = self.operator_cloud.list_users()
        self.assertIsNotNone(users)
        self.assertNotEqual([], users)

    def test_get_user(self):
        user = self.operator_cloud.get_user('admin')
        self.assertIsNotNone(user)
        self.assertIn('id', user)
        self.assertIn('name', user)
        self.assertEqual('admin', user['name'])

    def test_search_users(self):
        users = self.operator_cloud.search_users(filters={'is_enabled': True})
        self.assertIsNotNone(users)

    def test_search_users_jmespath(self):
        users = self.operator_cloud.search_users(filters="[?enabled]")
        self.assertIsNotNone(users)

    def test_create_user(self):
        user_name = self.user_prefix + '_create'
        user_email = 'nobody@nowhere.com'
        user = self._create_user(name=user_name, email=user_email)
        self.assertIsNotNone(user)
        self.assertEqual(user_name, user['name'])
        self.assertEqual(user_email, user['email'])
        self.assertTrue(user['is_enabled'])

    def test_delete_user(self):
        user_name = self.user_prefix + '_delete'
        user_email = 'nobody@nowhere.com'
        user = self._create_user(name=user_name, email=user_email)
        self.assertIsNotNone(user)
        self.assertTrue(self.operator_cloud.delete_user(user['id']))

    def test_delete_user_not_found(self):
        self.assertFalse(self.operator_cloud.delete_user('does_not_exist'))

    def test_update_user(self):
        user_name = self.user_prefix + '_updatev3'
        user_email = 'nobody@nowhere.com'
        user = self._create_user(name=user_name, email=user_email)
        self.assertIsNotNone(user)
        self.assertTrue(user['is_enabled'])

        # Pass some keystone v3 params. This should work no matter which
        # version of keystone we are testing against.
        new_user = self.operator_cloud.update_user(
            user['id'],
            name=user_name + '2',
            email='somebody@nowhere.com',
            enabled=False,
            password='secret',
            description='',
        )
        self.assertIsNotNone(new_user)
        self.assertEqual(user['id'], new_user['id'])
        self.assertEqual(user_name + '2', new_user['name'])
        self.assertEqual('somebody@nowhere.com', new_user['email'])
        self.assertFalse(new_user['is_enabled'])

    def test_update_user_password(self):
        user_name = self.user_prefix + '_password'
        user_email = 'nobody@nowhere.com'
        user = self._create_user(
            name=user_name, email=user_email, password='old_secret'
        )
        self.assertIsNotNone(user)
        self.assertTrue(user['enabled'])

        # This should work for both v2 and v3
        new_user = self.operator_cloud.update_user(
            user['id'], password='new_secret'
        )
        self.assertIsNotNone(new_user)
        self.assertEqual(user['id'], new_user['id'])
        self.assertEqual(user_name, new_user['name'])
        self.assertEqual(user_email, new_user['email'])
        self.assertTrue(new_user['enabled'])
        self.assertTrue(
            self.operator_cloud.grant_role(
                'member', user=user['id'], project='demo', wait=True
            )
        )
        self.addCleanup(
            self.operator_cloud.revoke_role,
            'member',
            user=user['id'],
            project='demo',
            wait=True,
        )

        new_cloud = self.operator_cloud.connect_as(
            user_id=user['id'], password='new_secret', project_name='demo'
        )

        self.assertIsNotNone(new_cloud)
        location = new_cloud.current_location
        self.assertEqual(location['project']['name'], 'demo')
        self.assertIsNotNone(new_cloud.service_catalog)

    def test_users_and_groups(self):
        group_name = self.getUniqueString('group')
        self.addCleanup(self.operator_cloud.delete_group, group_name)

        # Create a group
        group = self.operator_cloud.create_group(group_name, 'test group')
        self.assertIsNotNone(group)

        # Create a user
        user_name = self.user_prefix + '_ug'
        user_email = 'nobody@nowhere.com'
        user = self._create_user(name=user_name, email=user_email)
        self.assertIsNotNone(user)

        # Add the user to the group
        self.operator_cloud.add_user_to_group(user_name, group_name)
        self.assertTrue(
            self.operator_cloud.is_user_in_group(user_name, group_name)
        )

        # Remove them from the group
        self.operator_cloud.remove_user_from_group(user_name, group_name)
        self.assertFalse(
            self.operator_cloud.is_user_in_group(user_name, group_name)
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from openstack.network.v2 import (
    qos_dscp_marking_rule as _qos_dscp_marking_rule,
)
from openstack.tests.functional import base


class TestQoSDSCPMarkingRule(base.BaseFunctionalTest):
    QOS_POLICY_ID: str
    RULE_ID: str
    QOS_IS_SHARED = False
    QOS_POLICY_DESCRIPTION = "QoS policy description"
    RULE_DSCP_MARK = 36
    RULE_DSCP_MARK_NEW = 40

    def setUp(self):
        super().setUp()

        if not self.operator_cloud:
            self.skipTest("Operator cloud is required for this test")

        # Skip the tests if qos extension is not enabled.
        if not self.operator_cloud.network.find_extension("qos"):
            self.skipTest("Network qos extension disabled")

        self.QOS_POLICY_NAME = self.getUniqueString()
        self.RULE_ID = self.getUniqueString()
        qos_policy = self.operator_cloud.network.create_qos_policy(
            description=self.QOS_POLICY_DESCRIPTION,
            name=self.QOS_POLICY_NAME,
            shared=self.QOS_IS_SHARED,
        )
        self.QOS_POLICY_ID = qos_policy.id
        qos_rule = self.operator_cloud.network.create_qos_dscp_marking_rule(
            self.QOS_POLICY_ID,
            dscp_mark=self.RULE_DSCP_MARK,
        )
        assert isinstance(qos_rule, _qos_dscp_marking_rule.QoSDSCPMarkingRule)
        self.assertEqual(self.RULE_DSCP_MARK, qos_rule.dscp_mark)
        self.RULE_ID = qos_rule.id

    def tearDown(self):
        rule = self.operator_cloud.network.delete_qos_dscp_marking_rule(
            self.RULE_ID, self.QOS_POLICY_ID
        )
        qos_policy = self.operator_cloud.network.delete_qos_policy(
            self.QOS_POLICY_ID
        )
        self.assertIsNone(rule)
        self.assertIsNone(qos_policy)
        super().tearDown()

    def test_find(self):
        sot = self.operator_cloud.network.find_qos_dscp_marking_rule(
            self.RULE_ID, self.QOS_POLICY_ID, ignore_missing=False
        )
        self.assertEqual(self.RULE_ID, sot.id)
        self.assertEqual(self.RULE_DSCP_MARK, sot.dscp_mark)

    def test_get(self):
        sot = self.operator_cloud.network.get_qos_dscp_marking_rule(
            self.RULE_ID, self.QOS_POLICY_ID
        )
        self.assertEqual(self.RULE_ID, sot.id)
        self.assertEqual(self.QOS_POLICY_ID, sot.qos_policy_id)
        self.assertEqual(self.RULE_DSCP_MARK, sot.dscp_mark)

    def test_list(self):
        rule_ids = [
            o.id
            for o in self.operator_cloud.network.qos_dscp_marking_rules(
                self.QOS_POLICY_ID
            )
        ]
        self.assertIn(self.RULE_ID, rule_ids)

    def test_update(self):
        sot = self.operator_cloud.network.update_qos_dscp_marking_rule(
            self.RULE_ID, self.QOS_POLICY_ID, dscp_mark=self.RULE_DSCP_MARK_NEW
        )
        self.assertEqual(self.RULE_DSCP_MARK_NEW, sot.dscp_mark)



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
test_floating_ip_common
----------------------------------

Tests floating IP resource methods for Neutron and Nova-network.
"""

from unittest.mock import patch

from openstack.cloud import meta
from openstack.compute.v2 import server as _server
from openstack import connection
from openstack.tests import fakes
from openstack.tests.unit import base


class TestFloatingIP(base.TestCase):
    @patch.object(connection.Connection, 'get_floating_ip')
    @patch.object(connection.Connection, '_attach_ip_to_server')
    @patch.object(connection.Connection, 'available_floating_ip')
    def test_add_auto_ip(
        self,
        mock_available_floating_ip,
        mock_attach_ip_to_server,
        mock_get_floating_ip,
    ):
        server_dict = fakes.make_fake_server(
            server_id='server-id',
            name='test-server',
            status="ACTIVE",
            addresses={},
        )
        floating_ip_dict = {
            "id": "this-is-a-floating-ip-id",
            "fixed_ip_address": None,
            "internal_network": None,
            "floating_ip_address": "203.0.113.29",
            "network": "this-is-a-net-or-pool-id",
            "attached": False,
            "status": "ACTIVE",
        }

        mock_available_floating_ip.return_value = floating_ip_dict

        self.cloud.add_auto_ip(server=server_dict)

        mock_attach_ip_to_server.assert_called_with(
            timeout=60,
            wait=False,
            server=server_dict,
            floating_ip=floating_ip_dict,
            skip_attach=False,
        )

    @patch.object(connection.Connection, '_add_ip_from_pool')
    def test_add_ips_to_server_pool(self, mock_add_ip_from_pool):
        server_dict = fakes.make_fake_server(
            server_id='romeo',
            name='test-server',
            status="ACTIVE",
            addresses={},
        )
        pool = 'nova'

        self.cloud.add_ips_to_server(server_dict, ip_pool=pool)

        mock_add_ip_from_pool.assert_called_with(
            server_dict,
            pool,
            reuse=True,
            wait=False,
            timeout=60,
            fixed_address=None,
            nat_destination=None,
        )

    @patch.object(connection.Connection, 'has_service')
    @patch.object(connection.Connection, 'get_floating_ip')
    @patch.object(connection.Connection, '_add_auto_ip')
    def test_add_ips_to_server_ipv6_only(
        self, mock_add_auto_ip, mock_get_floating_ip, mock_has_service
    ):
        self.cloud._floating_ip_source = None
        self.cloud.force_ipv4 = False
        self.cloud._local_ipv6 = True
        mock_has_service.return_value = False
        server = fakes.make_fake_server(
            server_id='server-id',
            name='test-server',
            status="ACTIVE",
            addresses={
                'private': [{'addr': "10.223.160.141", 'version': 4}],
                'public': [
                    {
                        'OS-EXT-IPS-MAC:mac_addr': 'fa:16:3e:ae:7d:42',
                        'OS-EXT-IPS:type': 'fixed',
                        'addr': "2001:4800:7819:103:be76:4eff:fe05:8525",
                        'version': 6,
                    }
                ],
            },
        )
        server_dict = meta.add_server_interfaces(
            self.cloud, _server.Server(**server)
        )

        new_server = self.cloud.add_ips_to_server(server=server_dict)
        mock_get_floating_ip.assert_not_called()
        mock_add_auto_ip.assert_not_called()
        self.assertEqual(
            new_server['interface_ip'],
            '2001:4800:7819:103:be76:4eff:fe05:8525',
        )
        self.assertEqual(new_server['private_v4'], '10.223.160.141')
        self.assertEqual(new_server['public_v4'], '')
        self.assertEqual(
            new_server['public_v6'], '2001:4800:7819:103:be76:4eff:fe05:8525'
        )

    @patch.object(connection.Connection, 'has_service')
    @patch.object(connection.Connection, 'get_floating_ip')
    @patch.object(connection.Connection, '_add_auto_ip')
    def test_add_ips_to_server_rackspace(
        self, mock_add_auto_ip, mock_get_floating_ip, mock_has_service
    ):
        self.cloud._floating_ip_source = None
        self.cloud.force_ipv4 = False
        self.cloud._local_ipv6 = True
        mock_has_service.return_value = False
        server = fakes.make_fake_server(
            server_id='server-id',
            name='test-server',
            status="ACTIVE",
            addresses={
                'private': [{'addr': "10.223.160.141", 'version': 4}],
                'public': [
                    {'addr': "104.130.246.91", 'version': 4},
                    {
                        'addr': "2001:4800:7819:103:be76:4eff:fe05:8525",
                        'version': 6,
                    },
                ],
            },
        )
        server_dict = meta.add_server_interfaces(
            self.cloud, _server.Server(**server)
        )

        new_server = self.cloud.add_ips_to_server(server=server_dict)
        mock_get_floating_ip.assert_not_called()
        mock_add_auto_ip.assert_not_called()
        self.assertEqual(
            new_server['interface_ip'],
            '2001:4800:7819:103:be76:4eff:fe05:8525',
        )

    @patch.object(connection.Connection, 'has_service')
    @patch.object(connection.Connection, 'get_floating_ip')
    @patch.object(connection.Connection, '_add_auto_ip')
    def test_add_ips_to_server_rackspace_local_ipv4(
        self, mock_add_auto_ip, mock_get_floating_ip, mock_has_service
    ):
        self.cloud._floating_ip_source = None
        self.cloud.force_ipv4 = False
        self.cloud._local_ipv6 = False
        mock_has_service.return_value = False
        server = fakes.make_fake_server(
            server_id='server-id',
            name='test-server',
            status="ACTIVE",
            addresses={
                'private': [{'addr': "10.223.160.141", 'version': 4}],
                'public': [
                    {'addr': "104.130.246.91", 'version': 4},
                    {
                        'addr': "2001:4800:7819:103:be76:4eff:fe05:8525",
                        'version': 6,
                    },
                ],
            },
        )
        server_dict = meta.add_server_interfaces(
            self.cloud, _server.Server(**server)
        )

        new_server = self.cloud.add_ips_to_server(server=server_dict)
        mock_get_floating_ip.assert_not_called()
        mock_add_auto_ip.assert_not_called()
        self.assertEqual(new_server['interface_ip'], '104.130.246.91')

    @patch.object(connection.Connection, 'add_ip_list')
    def test_add_ips_to_server_ip_list(self, mock_add_ip_list):
        server_dict = fakes.make_fake_server(
            server_id='server-id',
            name='test-server',
            status="ACTIVE",
            addresses={},
        )
        ips = ['203.0.113.29', '172.24.4.229']

        self.cloud.add_ips_to_server(server_dict, ips=ips)

        mock_add_ip_list.assert_called_with(
            server_dict,
            ips,
            wait=False,
            timeout=60,
            fixed_address=None,
            nat_destination=None,
        )

    @patch.object(connection.Connection, '_needs_floating_ip')
    @patch.object(connection.Connection, '_add_auto_ip')
    def test_add_ips_to_server_auto_ip(
        self, mock_add_auto_ip, mock_needs_floating_ip
    ):
        server_dict = fakes.make_fake_server(
            server_id='server-id',
            name='test-server',
            status="ACTIVE",
            addresses={},
        )

        # TODO(mordred) REMOVE THIS MOCK WHEN THE NEXT PATCH LANDS
        # SERIOUSLY THIS TIME. NEXT PATCH - WHICH SHOULD ADD MOCKS FOR
        # list_ports AND list_networks AND list_subnets. BUT THAT WOULD
        # BE NOT ACTUALLY RELATED TO THIS PATCH. SO DO IT NEXT PATCH
        mock_needs_floating_ip.return_value = True

        self.cloud.add_ips_to_server(server_dict)

        mock_add_auto_ip.assert_called_with(
            server_dict, wait=False, timeout=60, reuse=True
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from typing import Any, cast
import warnings

from keystoneauth1 import adapter
from typing_extensions import Self

from openstack.common import metadata
from openstack import exceptions
from openstack import resource
from openstack import utils
from openstack import warnings as os_warnings


class Backup(resource.Resource, metadata.MetadataMixin):
    """Volume Backup"""

    resource_key = "backup"
    resources_key = "backups"
    base_path = "/backups"

    # TODO(gtema): Starting from ~3.31(3.45) Cinder seems to support also fuzzy
    # search (name~, status~, volume_id~). But this is not documented
    # officially and seem to require microversion be set
    _query_mapping = resource.QueryParameters(
        "limit",
        "marker",
        "offset",
        "project_id",
        "name",
        "status",
        "volume_id",
        "sort_key",
        "sort_dir",
        "sort",
        all_projects="all_tenants",
    )

    # capabilities
    allow_fetch = True
    allow_create = True
    allow_delete = True
    allow_commit = True
    allow_list = True

    #: Properties
    #: backup availability zone
    availability_zone = resource.Body("availability_zone")
    #: The container backup in
    container = resource.Body("container")
    #: The date and time when the resource was created.
    created_at = resource.Body("created_at")
    #: data timestamp
    #: The time when the data on the volume was first saved.
    #: If it is a backup from volume, it will be the same as created_at
    #: for a backup. If it is a backup from a snapshot,
    #: it will be the same as created_at for the snapshot.
    data_timestamp = resource.Body('data_timestamp')
    #: backup description
    description = resource.Body("description")
    #: The UUID of the encryption key. Only included for encrypted volumes.
    encryption_key_id = resource.Body("encryption_key_id")
    #: Backup fail reason
    fail_reason = resource.Body("fail_reason")
    #: Force backup
    force = resource.Body("force", type=bool)
    #: has_dependent_backups
    #: If this value is true, there are other backups depending on this backup.
    has_dependent_backups = resource.Body('has_dependent_backups', type=bool)
    #: Indicates whether the backup mode is incremental.
    #: If this value is true, the backup mode is incremental.
    #: If this value is false, the backup mode is full.
    is_incremental = resource.Body("is_incremental", type=bool)
    #: A list of links associated with this volume. *Type: list*
    links = resource.Body("links", type=list)
    #: The backup metadata. New in version 3.43
    metadata = resource.Body('metadata', type=dict)
    #: backup name
    name = resource.Body("name")
    #: backup object count
    object_count = resource.Body("object_count", type=int)
    #: The UUID of the owning project.
    #: New in version 3.18
    project_id = resource.Body('os-backup-project-attr:project_id')
    #: The size of the volume, in gibibytes (GiB).
    size = resource.Body("size", type=int)
    #: The UUID of the source volume snapshot.
    snapshot_id = resource.Body("snapshot_id")
    #: backup status
    #: values: creating, available, deleting, error, restoring, error_restoring
    status = resource.Body("status")
    #: The date and time when the resource was updated.
    updated_at = resource.Body("updated_at")
    #: The UUID of the project owner. New in 3.56
    user_id = resource.Body('user_id')
    #: The UUID of the volume.
    volume_id = resource.Body("volume_id")
    #: The name of the volume.
    volume_name = resource.Body("volume_name")

    _max_microversion = "3.64"

    def create(
        self,
        session: adapter.Adapter,
        prepend_key: bool = True,
        base_path: str | None = None,
        *,
        resource_request_key: str | None = None,
        resource_response_key: str | None = None,
        microversion: str | None = None,
        **params: Any,
    ) -> Self:
        """Create a remote resource based on this instance.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param prepend_key: A boolean indicating whether the resource_key
                            should be prepended in a resource creation
                            request. Default to True.
        :param str base_path: Base part of the URI for creating resources, if
                              different from
                              :data:`~openstack.resource.Resource.base_path`.
        :param dict params: Additional params to pass.
        :return: This :class:`Resource` instance.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
                 :data:`Resource.allow_create` is not set to ``True``.
        """
        if not self.allow_create:
            raise exceptions.MethodNotSupported(self, "create")

        session = self._get_session(session)
        microversion = self._get_microversion(session)
        requires_id = (
            self.create_requires_id
            if self.create_requires_id is not None
            else self.create_method == 'PUT'
        )

        if self.create_exclude_id_from_body:
            self._body._dirty.discard("id")

        if self.create_method == 'POST':
            request = self._prepare_request(
                requires_id=requires_id,
                prepend_key=prepend_key,
                base_path=base_path,
            )
            # NOTE(gtema) this is a funny example of when attribute
            # is called "incremental" on create, "is_incremental" on get
            # and use of "alias" or "aka" is not working for such conflict,
            # since our preferred attr name is exactly "is_incremental"
            body = cast(dict[str, Any], request.body)
            if 'is_incremental' in body['backup']:
                body['backup']['incremental'] = body['backup'].pop(
                    'is_incremental'
                )
            response = session.post(
                request.url,
                json=request.body,
                headers=request.headers,
                microversion=microversion,
                params=params,
            )
        else:
            # Just for safety of the implementation (since PUT removed)
            raise exceptions.ResourceFailure(
                f"Invalid create method: {self.create_method}"
            )

        has_body = (
            self.has_body
            if self.create_returns_body is None
            else self.create_returns_body
        )
        self.microversion = microversion
        self._translate_response(response, has_body=has_body)
        # direct comparision to False since we need to rule out None
        if self.has_body and self.create_returns_body is False:
            # fetch the body if it's required but not returned by create
            return self.fetch(session)
        return self

    def _action(self, session, body, microversion=None):
        """Preform backup actions given the message body."""
        url = utils.urljoin(self.base_path, self.id, 'action')
        resp = session.post(
            url, json=body, microversion=self._max_microversion
        )
        exceptions.raise_from_response(resp)
        return resp

    def export(self, session):
        """Export the current backup

        :param session: openstack session
        :return: The backup export record fields
        """
        url = utils.urljoin(self.base_path, self.id, "export_record")
        resp = session.get(url)
        exceptions.raise_from_response(resp)
        return resp

    def restore(self, session, volume_id=None, name=None):
        """Restore current backup to volume

        :param session: openstack session
        :param volume_id: The ID of the volume to restore the backup to.
        :param name: The name for new volume creation to restore.
        :return: Updated backup instance
        """
        url = utils.urljoin(self.base_path, self.id, "restore")
        body: dict[str, dict[str, Any]] = {'restore': {}}
        if volume_id:
            body['restore']['volume_id'] = volume_id
        if name:
            body['restore']['name'] = name
        if not (volume_id or name):
            raise exceptions.SDKException(
                'Either of `name` or `volume_id` must be specified.'
            )
        response = session.post(url, json=body)
        self._translate_response(response, resource_response_key='restore')
        return self

    def force_delete(self, session):
        """Force backup deletion"""
        body = {'os-force_delete': None}
        self._action(session, body)

    def reset_status(self, session, status):
        """Reset the status of the backup"""
        body = {'os-reset_status': {'status': status}}
        self._action(session, body)

    def reset(self, session, status):
        warnings.warn(
            "reset is a deprecated alias for reset_status and will be "
            "removed in a future release.",
            os_warnings.RemovedInSDK60Warning,
        )
        self.reset_status(session, status)


BackupDetail = Backup



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.block_storage.v3 import _proxy as _block_storage_v3
from openstack.tests.functional import base
from openstack import utils


class BaseBlockStorageTest(base.BaseFunctionalTest):
    _wait_for_timeout_key = 'OPENSTACKSDK_FUNC_TEST_TIMEOUT_BLOCK_STORAGE'

    admin_block_storage_client: _block_storage_v3.Proxy
    block_storage_client: _block_storage_v3.Proxy

    def setUp(self):
        super().setUp()
        self._set_user_cloud(block_storage_api_version='3')
        if not self.user_cloud.has_service('block-storage', '3'):
            self.skipTest('block-storage service not supported by cloud')

        self.admin_block_storage_client = utils.ensure_service_version(
            self.operator_cloud.block_storage, '3'
        )
        self.block_storage_client = utils.ensure_service_version(
            self.user_cloud.block_storage, '3'
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import warnings

from openstack import exceptions
from openstack import resource
from openstack import utils
from openstack import warnings as os_warnings


class Group(resource.Resource):
    resource_key = "group"
    resources_key = "groups"
    base_path = "/groups"

    # capabilities
    allow_fetch = True
    allow_create = True
    allow_delete = True
    allow_commit = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        "limit",
        "marker",
        "offset",
        "sort_dir",
        "sort_key",
        "sort",
        all_projects="all_tenants",
    )

    availability_zone = resource.Body("availability_zone")
    created_at = resource.Body("created_at")
    description = resource.Body("description")
    group_snapshot_id = resource.Body("group_snapshot_id")
    group_type = resource.Body("group_type")
    project_id = resource.Body("project_id")
    replication_targets = resource.Body("replication_targets", type=list)
    replication_status = resource.Body("replication_status")
    source_group_id = resource.Body("source_group_id")
    status = resource.Body("status")
    volumes = resource.Body("volumes", type=list)
    volume_types = resource.Body("volume_types", type=list)

    _max_microversion = "3.38"

    def _action(self, session, body):
        """Preform group actions given the message body."""
        session = self._get_session(session)
        microversion = self._get_microversion(session)
        url = utils.urljoin(self.base_path, self.id, 'action')
        response = session.post(url, json=body, microversion=microversion)
        exceptions.raise_from_response(response)
        return response

    def delete(self, session, *args, delete_volumes=False, **kwargs):
        """Delete a group."""
        body = {'delete': {'delete-volumes': delete_volumes}}
        self._action(session, body)

    def fetch_replication_targets(self, session):
        """Fetch replication targets for the group.

        :param session: The session to use for making this request.
        :return: This group with the ``replication_targets`` field populated.
        """
        body = {'list_replication_targets': None}
        response = self._action(session, body)
        self._body.attributes.update(
            {'replication_targets': response.json()['replication_targets']}
        )
        return self

    def enable_replication(self, session):
        """Enable replication for the group.

        :param session: The session to use for making this request.
        """
        body = {'enable_replication': None}
        self._action(session, body)

    def disable_replication(self, session):
        """Disable replication for the group.

        :param session: The session to use for making this request.
        """
        body = {'disable_replication': None}
        self._action(session, body)

    def failover_replication(
        self,
        session,
        *,
        allowed_attached_volume=False,
        secondary_backend_id=None,
    ):
        """Failover replication for the group.

        :param session: The session to use for making this request.
        :param allowed_attached_volume: Whether to allow attached volumes in
            the group.
        :param secondary_backend_id: The secondary backend ID.
        :returns: None
        """
        body = {
            'modify_body_for_action': {
                'allow_attached_volume': allowed_attached_volume,
                'secondary_backend_id': secondary_backend_id,
            },
        }
        self._action(session, body)

    def reset_status(self, session, status):
        """Resets the status for a group.

        :param session: The session to use for making this request.
        :param status: The status for the group.
        """
        body = {'reset_status': {'status': status}}
        self._action(session, body)

    def reset(self, session, status):
        warnings.warn(
            "reset is a deprecated alias for reset_status and will be "
            "removed in a future release.",
            os_warnings.RemovedInSDK60Warning,
        )
        self.reset_status(session, status)

    @classmethod
    def create_from_source(
        cls,
        session,
        group_snapshot_id,
        source_group_id,
        name=None,
        description=None,
    ):
        """Creates a new group from source."""
        session = cls._get_session(session)
        microversion = cls._get_microversion(session)
        url = utils.urljoin(cls.base_path, 'action')
        body = {
            'create-from-src': {
                'name': name,
                'description': description,
                'group_snapshot_id': group_snapshot_id,
                'source_group_id': source_group_id,
            }
        }
        response = session.post(url, json=body, microversion=microversion)
        exceptions.raise_from_response(response)

        group = Group()
        group._translate_response(response)
        return group



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.image.v2 import metadef_resource_type
from openstack.tests.unit import base


EXAMPLE = {
    "name": "OS::Cinder::Volume",
    "prefix": "CIM_PASD_",
    "properties_target": "image",
    "created_at": "2022-07-09T04:10:38Z",
}


class TestMetadefResourceTypeAssociation(base.TestCase):
    def test_basic(self):
        sot = metadef_resource_type.MetadefResourceTypeAssociation()
        self.assertIsNone(sot.resource_key)
        self.assertEqual('resource_type_associations', sot.resources_key)
        self.assertEqual(
            '/metadefs/namespaces/%(namespace_name)s/resource_types',
            sot.base_path,
        )
        self.assertTrue(sot.allow_create)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = metadef_resource_type.MetadefResourceTypeAssociation(**EXAMPLE)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE['prefix'], sot.prefix)
        self.assertEqual(EXAMPLE['properties_target'], sot.properties_target)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.image.v1 import _proxy as _proxy_v1
from openstack.image.v2 import _proxy as _proxy_v2
from openstack import service_description


class ImageService(
    service_description.ServiceDescription[_proxy_v1.Proxy | _proxy_v2.Proxy]
):
    """The image service."""

    supported_versions = {
        '1': _proxy_v1.Proxy,
        '2': _proxy_v2.Proxy,
    }



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from openstack.network.v2 import network
from openstack.network.v2 import port
from openstack.network.v2 import subnet
from openstack.tests.functional import base


class TestNetworkIPAvailability(base.BaseFunctionalTest):
    IPV4 = 4
    CIDR = "10.100.0.0/24"
    NET_ID: str
    SUB_ID: str
    PORT_ID: str

    def setUp(self):
        super().setUp()
        if not self.operator_cloud:
            self.skipTest("Operator cloud required for this test")
        if not self.operator_cloud._has_neutron_extension(
            "network-ip-availability"
        ):
            self.skipTest(
                "Neutron network-ip-availability extension is required "
                "for this test"
            )

        self.NET_NAME = self.getUniqueString()
        self.SUB_NAME = self.getUniqueString()
        self.PORT_NAME = self.getUniqueString()
        self.UPDATE_NAME = self.getUniqueString()
        net = self.operator_cloud.network.create_network(name=self.NET_NAME)
        assert isinstance(net, network.Network)
        self.assertEqual(self.NET_NAME, net.name)
        self.NET_ID = net.id
        sub = self.operator_cloud.network.create_subnet(
            name=self.SUB_NAME,
            ip_version=self.IPV4,
            network_id=self.NET_ID,
            cidr=self.CIDR,
        )
        assert isinstance(sub, subnet.Subnet)
        self.assertEqual(self.SUB_NAME, sub.name)
        self.SUB_ID = sub.id
        prt = self.operator_cloud.network.create_port(
            name=self.PORT_NAME, network_id=self.NET_ID
        )
        assert isinstance(prt, port.Port)
        self.assertEqual(self.PORT_NAME, prt.name)
        self.PORT_ID = prt.id

    def tearDown(self):
        sot = self.operator_cloud.network.delete_port(self.PORT_ID)
        self.assertIsNone(sot)
        sot = self.operator_cloud.network.delete_subnet(self.SUB_ID)
        self.assertIsNone(sot)
        sot = self.operator_cloud.network.delete_network(self.NET_ID)
        self.assertIsNone(sot)
        super().tearDown()

    def test_find(self):
        sot = self.operator_cloud.network.find_network_ip_availability(
            self.NET_ID, ignore_missing=False
        )
        self.assertEqual(self.NET_ID, sot.network_id)

    def test_get(self):
        sot = self.operator_cloud.network.get_network_ip_availability(
            self.NET_ID
        )
        self.assertEqual(self.NET_ID, sot.network_id)
        self.assertEqual(self.NET_NAME, sot.network_name)

    def test_list(self):
        ids = [
            o.network_id
            for o in self.operator_cloud.network.network_ip_availabilities()
        ]
        self.assertIn(self.NET_ID, ids)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from openstack.network.v2 import (
    qos_packet_rate_limit_rule as _qos_packet_rate_limit_rule,
)
from openstack.tests.functional import base


class TestQoSPacketRateLimitRule(base.BaseFunctionalTest):
    QOS_POLICY_ID: str
    RULE_ID: str
    QOS_IS_SHARED = False
    QOS_POLICY_DESCRIPTION = 'QoS policy description'
    RULE_MAX_KPPS = 1500
    RULE_MAX_KPPS_NEW = 1800
    RULE_MAX_BURST_KPPS = 1100
    RULE_MAX_BURST_KPPS_NEW = 1300
    RULE_DIRECTION = 'egress'
    RULE_DIRECTION_NEW = 'ingress'
    RULE_DIRECTION_NEW_2 = 'any'

    def setUp(self):
        super().setUp()

        if not self.operator_cloud:
            self.skipTest('Operator cloud required for this test')

        # Skip the tests if qos-bw-limit-direction extension is not enabled.
        if not self.operator_cloud.network.find_extension('qos-pps'):
            self.skipTest("Network qos-pps extension disabled")

        self.QOS_POLICY_NAME = self.getUniqueString()
        self.RULE_ID = self.getUniqueString()
        qos_policy = self.operator_cloud.network.create_qos_policy(
            description=self.QOS_POLICY_DESCRIPTION,
            name=self.QOS_POLICY_NAME,
            shared=self.QOS_IS_SHARED,
        )
        self.QOS_POLICY_ID = qos_policy.id
        qos_rule = (
            self.operator_cloud.network.create_qos_packet_rate_limit_rule(
                self.QOS_POLICY_ID,
                max_kpps=self.RULE_MAX_KPPS,
                max_burst_kpps=self.RULE_MAX_BURST_KPPS,
                direction=self.RULE_DIRECTION,
            )
        )
        assert isinstance(
            qos_rule, _qos_packet_rate_limit_rule.QoSPacketRateLimitRule
        )
        self.assertEqual(self.RULE_MAX_KPPS, qos_rule.max_kpps)
        self.assertEqual(self.RULE_MAX_BURST_KPPS, qos_rule.max_burst_kpps)
        self.assertEqual(self.RULE_DIRECTION, qos_rule.direction)
        self.RULE_ID = qos_rule.id

    def tearDown(self):
        rule = self.operator_cloud.network.delete_qos_packet_rate_limit_rule(
            self.RULE_ID, self.QOS_POLICY_ID
        )
        qos_policy = self.operator_cloud.network.delete_qos_policy(
            self.QOS_POLICY_ID
        )
        self.assertIsNone(rule)
        self.assertIsNone(qos_policy)
        super().tearDown()

    def test_find(self):
        sot = self.operator_cloud.network.find_qos_packet_rate_limit_rule(
            self.RULE_ID, self.QOS_POLICY_ID, ignore_missing=False
        )
        self.assertEqual(self.RULE_ID, sot.id)
        self.assertEqual(self.RULE_MAX_KPPS, sot.max_kpps)
        self.assertEqual(self.RULE_MAX_BURST_KPPS, sot.max_burst_kpps)
        self.assertEqual(self.RULE_DIRECTION, sot.direction)

    def test_get(self):
        sot = self.operator_cloud.network.get_qos_packet_rate_limit_rule(
            self.RULE_ID, self.QOS_POLICY_ID
        )
        self.assertEqual(self.RULE_ID, sot.id)
        self.assertEqual(self.QOS_POLICY_ID, sot.qos_policy_id)
        self.assertEqual(self.RULE_MAX_KPPS, sot.max_kpps)
        self.assertEqual(self.RULE_MAX_BURST_KPPS, sot.max_burst_kpps)
        self.assertEqual(self.RULE_DIRECTION, sot.direction)

    def test_list(self):
        rule_ids = [
            o.id
            for o in self.operator_cloud.network.qos_packet_rate_limit_rules(
                self.QOS_POLICY_ID
            )
        ]
        self.assertIn(self.RULE_ID, rule_ids)

    def test_update(self):
        sot = self.operator_cloud.network.update_qos_packet_rate_limit_rule(
            self.RULE_ID,
            self.QOS_POLICY_ID,
            max_kpps=self.RULE_MAX_KPPS_NEW,
            max_burst_kpps=self.RULE_MAX_BURST_KPPS_NEW,
            direction=self.RULE_DIRECTION_NEW,
        )
        self.assertEqual(self.RULE_MAX_KPPS_NEW, sot.max_kpps)
        self.assertEqual(self.RULE_MAX_BURST_KPPS_NEW, sot.max_burst_kpps)
        self.assertEqual(self.RULE_DIRECTION_NEW, sot.direction)
